@@ -1,17 +1,22 @@
 open! Import
 include Array0
-module Int = Int0
 
-let raise_s = Error.raise_s
+type 'a t = 'a array [@@deriving_inline compare ~localize, globalize, sexp, sexp_grammar]
+
+let compare__local : 'a. ('a -> 'a -> int) -> 'a t -> 'a t -> int = compare_array__local
+let compare : 'a. ('a -> 'a -> int) -> 'a t -> 'a t -> int = compare_array
+
+let globalize : 'a. ('a -> 'a) -> 'a t -> 'a t =
+  fun (type a__009_) : ((a__009_ -> a__009_) -> a__009_ t -> a__009_ t) -> globalize_array
+;;
+
+let t_of_sexp : 'a. (Sexplib0.Sexp.t -> 'a) -> Sexplib0.Sexp.t -> 'a t = array_of_sexp
+let sexp_of_t : 'a. ('a -> Sexplib0.Sexp.t) -> 'a t -> Sexplib0.Sexp.t = sexp_of_array
+
+let t_sexp_grammar : 'a. 'a Sexplib0.Sexp_grammar.t -> 'a t Sexplib0.Sexp_grammar.t =
+  fun _'a_sexp_grammar -> array_sexp_grammar _'a_sexp_grammar
+;;
 
-type 'a t = 'a array [@@deriving_inline compare, sexp]
-let compare : 'a . ('a -> 'a -> int) -> 'a t -> 'a t -> int = compare_array
-let t_of_sexp :
-  'a . (Ppx_sexp_conv_lib.Sexp.t -> 'a) -> Ppx_sexp_conv_lib.Sexp.t -> 'a t =
-  array_of_sexp
-let sexp_of_t :
-  'a . ('a -> Ppx_sexp_conv_lib.Sexp.t) -> 'a t -> Ppx_sexp_conv_lib.Sexp.t =
-  sexp_of_array
 [@@@end]
 
 (* This module implements a new in-place, constant heap sorting algorithm to replace the
@@ -42,11 +47,15 @@ let sexp_of_t :
      Slides at http://www.cs.princeton.edu/~rs/talks/QuicksortIsOptimal.pdf
    - http://www.sorting-algorithms.com/quick-sort-3-way *)
 
-module Sort = struct
-  (* For the sake of speed we could use unsafe get/set throughout, but speed tests don't
-     show a significant improvement. *)
-  let get = get
-  let set = set
+module Sorter (S : sig
+  type 'a t
+
+  val get : 'a t -> int -> 'a
+  val set : 'a t -> int -> 'a -> unit
+  val length : 'a t -> int
+end) =
+struct
+  include S
 
   let swap arr i j =
     let tmp = get arr i in
@@ -65,27 +74,26 @@ module Sort = struct
 
   (* http://en.wikipedia.org/wiki/Insertion_sort *)
   module Insertion_sort : Sort = struct
+    (* loop invariants:
+       1.  the subarray arr[left .. i-1] is sorted
+       2.  the subarray arr[i+1 .. pos] is sorted and contains only elements > v
+       3.  arr[i] may be thought of as containing v
+    *)
+    let rec insert_loop arr ~left ~compare i v =
+      let i_next = i - 1 in
+      if i_next >= left && compare (get arr i_next) v > 0
+      then (
+        set arr i (get arr i_next);
+        insert_loop arr ~left ~compare i_next v)
+      else i
+    ;;
+
     let sort arr ~compare ~left ~right =
       (* loop invariant:
          [arr] is sorted from [left] to [pos - 1], inclusive *)
       for pos = left + 1 to right do
-        (* loop invariants:
-           1.  the subarray arr[left .. i-1] is sorted
-           2.  the subarray arr[i+1 .. pos] is sorted and contains only elements > v
-           3.  arr[i] may be thought of as containing v
-
-           Note that this does not allocate a closure, but is left in the for
-           loop for the readability of the documentation. *)
-        let rec loop arr ~left ~compare i v =
-          let i_next = i - 1 in
-          if i_next >= left && compare (get arr i_next) v > 0
-          then (
-            set arr i (get arr i_next);
-            loop arr ~left ~compare i_next v)
-          else i
-        in
         let v = get arr pos in
-        let final_pos = loop arr ~left ~compare pos v in
+        let final_pos = insert_loop arr ~left ~compare pos v in
         set arr final_pos v
       done
     ;;
@@ -151,7 +159,7 @@ module Sort = struct
       -> int
       -> unit
   end = struct
-    let five_element_sort arr ~compare m1 m2 m3 m4 m5 =
+    let five_element_sort arr ~(compare : _ -> _ -> _) m1 m2 m3 m4 m5 =
       let compare_and_swap i j =
         if compare (get arr i) (get arr j) > 0 then swap arr i j
       in
@@ -176,7 +184,7 @@ module Sort = struct
       compare_and_swap m3 m4;
       compare_and_swap m2 m5;
       compare_and_swap m2 m3;
-      compare_and_swap m4 m5
+      compare_and_swap m4 m5 [@nontail]
     ;;
 
     (* choose pivots for the array by sorting 5 elements and examining the center three
@@ -187,7 +195,7 @@ module Sort = struct
          by itself
          To this end we look at the center 3 elements of the 5 and return pairs of equal
          elements or the widest range *)
-    let choose_pivots arr ~compare ~left ~right =
+    let choose_pivots arr ~(compare : _ -> _ -> _) ~left ~right =
       let sixth = (right - left) / 6 in
       let m1 = left + sixth in
       let m2 = m1 + sixth in
@@ -205,7 +213,7 @@ module Sort = struct
       else m2_val, m4_val, false
     ;;
 
-    let dual_pivot_partition arr ~compare ~left ~right =
+    let dual_pivot_partition arr ~(compare : _ -> _ -> _) ~left ~right =
       let pivot1, pivot2, pivots_equal = choose_pivots arr ~compare ~left ~right in
       (* loop invariants:
          1.  left <= l < r <= right
@@ -252,54 +260,118 @@ module Sort = struct
         intro_sort arr ~max_depth ~compare ~left:(r + 1) ~right)
     ;;
 
-    let log10_of_3 = Caml.log10 3.
-    let log3 x = Caml.log10 x /. log10_of_3
-
     let sort arr ~compare ~left ~right =
-      let len = right - left + 1 in
       let heap_sort_switch_depth =
-        (* with perfect 3-way partitioning, this is the recursion depth *)
-        Int.of_float (log3 (Int.to_float len))
+        (* We bail out to heap sort at a recursion depth of 32. GNU introsort uses 2lg(n).
+           The expected recursion depth for perfect 3-way splits is log_3(n).
+
+           Using 32 means a balanced 3-way split would work up to 3^32 elements (roughly
+           2^50 or 10^15). GNU reaches a depth of 32 at 65536 elements.
+
+           For small arrays, this makes us less likely to bail out to heap sort, but the
+           32*N cost before we do is not that much.
+
+           For large arrays, this means we are more likely to bail out to heap sort at
+           some point if we get some bad splits or if the array is huge. But that's only a
+           constant factor cost in the final stages of recursion.
+
+           All in all, this seems to be a small tradeoff and avoids paying a cost to
+           compute a logarithm at the start. *)
+        32
       in
       intro_sort arr ~max_depth:heap_sort_switch_depth ~compare ~left ~right
     ;;
   end
+
+  let sort ?pos ?len arr ~(compare : _ -> _ -> _) =
+    let pos, len =
+      Ordered_collection_common.get_pos_len_exn () ?pos ?len ~total_length:(length arr)
+    in
+    Intro_sort.sort arr ~compare ~left:pos ~right:(pos + len - 1)
+  ;;
 end
+[@@inline]
 
-let sort ?pos ?len arr ~compare =
-  let pos, len =
-    Ordered_collection_common.get_pos_len_exn () ?pos ?len ~total_length:(length arr)
-  in
-  Sort.Intro_sort.sort arr ~compare ~left:pos ~right:(pos + len - 1)
-;;
+module Sort = Sorter (struct
+  type nonrec 'a t = 'a t
 
+  let get = unsafe_get
+  let set = unsafe_set
+  let length = length
+end)
+
+let sort = Sort.sort
+let of_array t = t
 let to_array t = t
 let is_empty t = length t = 0
 
 let is_sorted t ~compare =
-  let rec is_sorted_loop t ~compare i =
-    if i < 1
-    then true
-    else compare t.(i - 1) t.(i) <= 0 && is_sorted_loop t ~compare (i - 1)
-  in
-  is_sorted_loop t ~compare (length t - 1)
+  let i = ref (length t - 1) in
+  let result = ref true in
+  while !i > 0 && !result do
+    let elt_i = unsafe_get t !i in
+    let elt_i_minus_1 = unsafe_get t (!i - 1) in
+    if compare elt_i_minus_1 elt_i > 0 then result := false;
+    decr i
+  done;
+  !result
 ;;
 
 let is_sorted_strictly t ~compare =
-  let rec is_sorted_strictly_loop t ~compare i =
-    if i < 1
-    then true
-    else compare t.(i - 1) t.(i) < 0 && is_sorted_strictly_loop t ~compare (i - 1)
-  in
-  is_sorted_strictly_loop t ~compare (length t - 1)
+  let i = ref (length t - 1) in
+  let result = ref true in
+  while !i > 0 && !result do
+    let elt_i = unsafe_get t !i in
+    let elt_i_minus_1 = unsafe_get t (!i - 1) in
+    if compare elt_i_minus_1 elt_i >= 0 then result := false;
+    decr i
+  done;
+  !result
 ;;
 
+let merge a1 a2 ~compare =
+  let l1 = Array.length a1 in
+  let l2 = Array.length a2 in
+  if l1 = 0
+  then copy a2
+  else if l2 = 0
+  then copy a1
+  else if compare (unsafe_get a2 0) (unsafe_get a1 (l1 - 1)) >= 0
+  then append a1 a2
+  else if compare (unsafe_get a1 0) (unsafe_get a2 (l2 - 1)) > 0
+  then append a2 a1
+  else (
+    let len = l1 + l2 in
+    let merged = create ~len (unsafe_get a1 0) in
+    let a1_index = ref 0 in
+    let a2_index = ref 0 in
+    for i = 0 to len - 1 do
+      let use_a1 =
+        if l1 = !a1_index
+        then false
+        else if l2 = !a2_index
+        then true
+        else compare (unsafe_get a1 !a1_index) (unsafe_get a2 !a2_index) <= 0
+      in
+      if use_a1
+      then (
+        unsafe_set merged i (unsafe_get a1 !a1_index);
+        a1_index := !a1_index + 1)
+      else (
+        unsafe_set merged i (unsafe_get a2 !a2_index);
+        a2_index := !a2_index + 1)
+    done;
+    merged)
+;;
+
+let copy_matrix = map ~f:copy
+
 let folding_map t ~init ~f =
   let acc = ref init in
   map t ~f:(fun x ->
     let new_acc, y = f !acc x in
     acc := new_acc;
-    y)
+    y) [@nontail]
 ;;
 
 let fold_map t ~init ~f =
@@ -314,17 +386,37 @@ let fold_map t ~init ~f =
 ;;
 
 let fold_result t ~init ~f = Container.fold_result ~fold ~init ~f t
-let fold_until t ~init ~f = Container.fold_until ~fold ~init ~f t
-let count t ~f = Container.count ~fold t ~f
+let fold_until t ~init ~f ~finish = Container.fold_until ~fold ~init ~f t ~finish
 let sum m t ~f = Container.sum ~fold m t ~f
-let min_elt t ~compare = Container.min_elt ~fold t ~compare
-let max_elt t ~compare = Container.max_elt ~fold t ~compare
+
+let[@inline always] extremal_element t ~compare ~keep_left_if =
+  if is_empty t
+  then None
+  else (
+    let result = ref (unsafe_get t 0) in
+    for i = 1 to length t - 1 do
+      let x = unsafe_get t i in
+      result := Bool.select ((keep_left_if [@inlined]) (compare x !result)) x !result
+    done;
+    Some !result)
+;;
+
+let min_elt t ~compare =
+  (extremal_element [@inlined]) t ~compare ~keep_left_if:(fun compare_result ->
+    compare_result < 0)
+;;
+
+let max_elt t ~compare =
+  (extremal_element [@inlined]) t ~compare ~keep_left_if:(fun compare_result ->
+    compare_result > 0)
+;;
 
 let foldi t ~init ~f =
-  let rec foldi_loop t i ac ~f =
-    if i = length t then ac else foldi_loop t (i + 1) (f i ac t.(i)) ~f
-  in
-  foldi_loop t 0 init ~f
+  let acc = ref init in
+  for i = 0 to length t - 1 do
+    acc := f i !acc (unsafe_get t i)
+  done;
+  !acc
 ;;
 
 let folding_mapi t ~init ~f =
@@ -332,7 +424,7 @@ let folding_mapi t ~init ~f =
   mapi t ~f:(fun i x ->
     let new_acc, y = f i !acc x in
     acc := new_acc;
-    y)
+    y) [@nontail]
 ;;
 
 let fold_mapi t ~init ~f =
@@ -346,8 +438,20 @@ let fold_mapi t ~init ~f =
   !acc, result
 ;;
 
+let count t ~f =
+  let result = ref 0 in
+  for i = 0 to Array.length t - 1 do
+    result := !result + (f (Array.unsafe_get t i) |> Bool.to_int)
+  done;
+  !result
+;;
+
 let counti t ~f =
-  foldi t ~init:0 ~f:(fun idx count a -> if f idx a then count + 1 else count)
+  let result = ref 0 in
+  for i = 0 to Array.length t - 1 do
+    result := !result + (f i (Array.unsafe_get t i) |> Bool.to_int)
+  done;
+  !result
 ;;
 
 let concat_map t ~f = concat (to_list (map ~f t))
@@ -363,6 +467,12 @@ let rev_inplace t =
   done
 ;;
 
+let rev t =
+  let t = copy t in
+  rev_inplace t;
+  t
+;;
+
 let of_list_rev l =
   match l with
   | [] -> [||]
@@ -395,7 +505,7 @@ let of_list_map xs ~f =
         unsafe_set a i (f hd);
         fill (i + 1) tl
     in
-    fill 1 tl
+    fill 1 tl [@nontail]
 ;;
 
 let of_list_mapi xs ~f =
@@ -409,7 +519,7 @@ let of_list_mapi xs ~f =
         unsafe_set a i (f i hd);
         fill a (i + 1) tl
     in
-    fill a 1 tl
+    fill a 1 tl [@nontail]
 ;;
 
 let of_list_rev_map xs ~f =
@@ -438,132 +548,166 @@ let filter_mapi t ~f =
   if !k = length t then !r else if !k > 0 then sub ~pos:0 ~len:!k !r else [||]
 ;;
 
-let filter_map t ~f = filter_mapi t ~f:(fun _i a -> f a)
+let filter_map t ~f = filter_mapi t ~f:(fun _i a -> f a) [@nontail]
 let filter_opt t = filter_map t ~f:Fn.id
 
+let raise_length_mismatch name n1 n2 =
+  invalid_argf "length mismatch in %s: %d <> %d" name n1 n2 ()
+  [@@cold] [@@inline never] [@@local never] [@@specialise never]
+;;
+
+let check_length2_exn name t1 t2 =
+  let n1 = length t1 in
+  let n2 = length t2 in
+  if n1 <> n2 then raise_length_mismatch name n1 n2
+;;
+
 let iter2_exn t1 t2 ~f =
-  if length t1 <> length t2 then invalid_arg "Array.iter2_exn";
-  iteri t1 ~f:(fun i x1 -> f x1 t2.(i))
+  check_length2_exn "Array.iter2_exn" t1 t2;
+  iteri t1 ~f:(fun i x1 -> f x1 (unsafe_get t2 i)) [@nontail]
 ;;
 
 let map2_exn t1 t2 ~f =
-  let len = length t1 in
-  if length t2 <> len then invalid_arg "Array.map2_exn";
-  init len ~f:(fun i -> f t1.(i) t2.(i))
+  check_length2_exn "Array.map2_exn" t1 t2;
+  init (length t1) ~f:(fun i -> f (unsafe_get t1 i) (unsafe_get t2 i)) [@nontail]
 ;;
 
 let fold2_exn t1 t2 ~init ~f =
-  if length t1 <> length t2 then invalid_arg "Array.fold2_exn";
-  foldi t1 ~init ~f:(fun i ac x -> f ac x t2.(i))
+  check_length2_exn "Array.fold2_exn" t1 t2;
+  foldi t1 ~init ~f:(fun i ac x -> f ac x (unsafe_get t2 i)) [@nontail]
 ;;
 
-let filter t ~f = filter_map t ~f:(fun x -> if f x then Some x else None)
-let filteri t ~f = filter_mapi t ~f:(fun i x -> if f i x then Some x else None)
+let filter t ~f = filter_map t ~f:(fun x -> if f x then Some x else None) [@nontail]
+let filteri t ~f = filter_mapi t ~f:(fun i x -> if f i x then Some x else None) [@nontail]
 
 let exists t ~f =
-  let rec exists_loop t ~f i =
-    if i < 0 then false else f t.(i) || exists_loop t ~f (i - 1)
-  in
-  exists_loop t ~f (length t - 1)
+  let i = ref (length t - 1) in
+  let result = ref false in
+  while !i >= 0 && not !result do
+    if f (unsafe_get t !i) then result := true else decr i
+  done;
+  !result
 ;;
 
 let existsi t ~f =
-  let rec existsi_loop t ~f i =
-    if i < 0 then false else f i t.(i) || existsi_loop t ~f (i - 1)
-  in
-  existsi_loop t ~f (length t - 1)
+  let i = ref (length t - 1) in
+  let result = ref false in
+  while !i >= 0 && not !result do
+    if f !i (unsafe_get t !i) then result := true else decr i
+  done;
+  !result
 ;;
 
-let mem t a ~equal = exists t ~f:(equal a)
+let mem t a ~equal = exists t ~f:(equal a) [@nontail]
 
 let for_all t ~f =
-  let rec for_all_loop t ~f i =
-    if i < 0 then true else f t.(i) && for_all_loop t ~f (i - 1)
-  in
-  for_all_loop t ~f (length t - 1)
+  let i = ref (length t - 1) in
+  let result = ref true in
+  while !i >= 0 && !result do
+    if not (f (unsafe_get t !i)) then result := false else decr i
+  done;
+  !result
 ;;
 
 let for_alli t ~f =
-  let rec for_alli_loop t ~f i =
-    if i < 0 then true else f i t.(i) && for_alli_loop t ~f (i - 1)
-  in
-  for_alli_loop t ~f (length t - 1)
+  let length = length t in
+  let i = ref (length - 1) in
+  let result = ref true in
+  while !i >= 0 && !result do
+    if not (f !i (unsafe_get t !i)) then result := false else decr i
+  done;
+  !result
 ;;
 
 let exists2_exn t1 t2 ~f =
-  let rec exists2_exn_loop t1 t2 ~f i =
-    if i < 0 then false else f t1.(i) t2.(i) || exists2_exn_loop t1 t2 ~f (i - 1)
-  in
-  let len = length t1 in
-  if length t2 <> len then invalid_arg "Array.exists2_exn";
-  exists2_exn_loop t1 t2 ~f (len - 1)
+  check_length2_exn "Array.exists2_exn" t1 t2;
+  let i = ref (length t1 - 1) in
+  let result = ref false in
+  while !i >= 0 && not !result do
+    if f (unsafe_get t1 !i) (unsafe_get t2 !i) then result := true else decr i
+  done;
+  !result
 ;;
 
-let for_all2_exn t1 t2 ~f =
-  let rec for_all2_loop t1 t2 ~f i =
-    if i < 0 then true else f t1.(i) t2.(i) && for_all2_loop t1 t2 ~f (i - 1)
-  in
-  let len = length t1 in
-  if length t2 <> len then invalid_arg "Array.for_all2_exn";
-  for_all2_loop t1 t2 ~f (len - 1)
+let for_all2_local_exn t1 t2 ~f =
+  check_length2_exn "Array.for_all2_exn" t1 t2;
+  let i = ref (length t1 - 1) in
+  let result = ref true in
+  while !i >= 0 && !result do
+    if not (f (unsafe_get t1 !i) (unsafe_get t2 !i)) then result := false else decr i
+  done;
+  !result
 ;;
 
-let equal equal t1 t2 = length t1 = length t2 && for_all2_exn t1 t2 ~f:equal
-
+let for_all2_exn t1 t2 ~f = for_all2_local_exn t1 t2 ~f
+let equal__local equal t1 t2 = length t1 = length t2 && for_all2_local_exn t1 t2 ~f:equal
+let equal equal t1 t2 = equal__local equal t1 t2
 
 let map_inplace t ~f =
   for i = 0 to length t - 1 do
-    t.(i) <- f t.(i)
+    unsafe_set t i (f (unsafe_get t i))
   done
 ;;
 
-let findi t ~f =
-  let rec findi_loop t ~f ~length i =
-    if i >= length
-    then None
-    else if f i t.(i)
-    then Some (i, t.(i))
-    else findi_loop t ~f ~length (i + 1)
-  in
+let[@inline always] findi_internal t ~f ~if_found ~if_not_found =
   let length = length t in
-  findi_loop t ~f ~length 0
+  if length = 0
+  then if_not_found ()
+  else (
+    let i = ref 0 in
+    let found = ref false in
+    let value_found = ref (unsafe_get t 0) in
+    while (not !found) && !i < length do
+      let value = unsafe_get t !i in
+      if f !i value
+      then (
+        value_found := value;
+        found := true)
+      else incr i
+    done;
+    if !found then if_found ~i:!i ~value:!value_found else if_not_found ())
 ;;
 
-let findi_exn =
-  let not_found = Not_found_s (Atom "Array.findi_exn: not found") in
-  let findi_exn t ~f =
-    match findi t ~f with
-    | None -> raise not_found
-    | Some x -> x
-  in
-  (* named to preserve symbol in compiled binary *)
-  findi_exn
+let findi t ~f =
+  findi_internal
+    t
+    ~f
+    ~if_found:(fun ~i ~value -> Some (i, value))
+    ~if_not_found:(fun () -> None)
 ;;
 
-let find_exn =
-  let not_found = Not_found_s (Atom "Array.find_exn: not found") in
-  let find_exn t ~f =
-    match findi t ~f:(fun _i x -> f x) with
-    | None -> raise not_found
-    | Some (_i, x) -> x
-  in
-  (* named to preserve symbol in compiled binary *)
-  find_exn
+let findi_exn t ~f =
+  findi_internal
+    t
+    ~f
+    ~if_found:(fun ~i ~value -> i, value)
+    ~if_not_found:(fun () -> raise (Not_found_s (Atom "Array.findi_exn: not found")))
+;;
+
+let find_exn t ~f =
+  findi_internal
+    t
+    ~f:(fun _i x -> f x)
+    ~if_found:(fun ~i:_ ~value -> value)
+    ~if_not_found:(fun () -> raise (Not_found_s (Atom "Array.find_exn: not found")))
+  [@nontail]
 ;;
 
 let find t ~f = Option.map (findi t ~f:(fun _i x -> f x)) ~f:(fun (_i, x) -> x)
 
 let find_map t ~f =
-  let rec find_map_loop t ~f ~length i =
-    if i >= length
-    then None
-    else (
-      match f t.(i) with
-      | None -> find_map_loop t ~f ~length (i + 1)
-      | Some _ as res -> res)
-  in
   let length = length t in
-  find_map_loop t ~f ~length 0
+  if length = 0
+  then None
+  else (
+    let i = ref 0 in
+    let value_found = ref None in
+    while Option.is_none !value_found && !i < length do
+      let value = unsafe_get t !i in
+      value_found := f value;
+      incr i
+    done;
+    !value_found)
 ;;
 
 let find_map_exn =
@@ -578,16 +722,18 @@ let find_map_exn =
 ;;
 
 let find_mapi t ~f =
-  let rec find_mapi_loop t ~f ~length i =
-    if i >= length
-    then None
-    else (
-      match f i t.(i) with
-      | None -> find_mapi_loop t ~f ~length (i + 1)
-      | Some _ as res -> res)
-  in
   let length = length t in
-  find_mapi_loop t ~f ~length 0
+  if length = 0
+  then None
+  else (
+    let i = ref 0 in
+    let value_found = ref None in
+    while Option.is_none !value_found && !i < length do
+      let value = unsafe_get t !i in
+      value_found := f !i value;
+      incr i
+    done;
+    !value_found)
 ;;
 
 let find_mapi_exn =
@@ -608,9 +754,9 @@ let find_consecutive_duplicate t ~equal =
   else (
     let result = ref None in
     let i = ref 1 in
-    let prev = ref t.(0) in
+    let prev = ref (unsafe_get t 0) in
     while !i < n do
-      let cur = t.(!i) in
+      let cur = unsafe_get t !i in
       if equal cur !prev
       then (
         result := Some (!prev, cur);
@@ -626,9 +772,9 @@ let reduce t ~f =
   if length t = 0
   then None
   else (
-    let r = ref t.(0) in
+    let r = ref (unsafe_get t 0) in
     for i = 1 to length t - 1 do
-      r := f !r t.(i)
+      r := f !r (unsafe_get t i)
     done;
     Some !r)
 ;;
@@ -684,29 +830,36 @@ let sorted_copy t ~compare =
   t1
 ;;
 
-let partitioni_tf t ~f =
-  let both = mapi t ~f:(fun i x -> if f i x then Either.First x else Either.Second x) in
-  let trues =
+let partition_mapi t ~f =
+  let (both : _ Either.t t) = mapi t ~f in
+  let firsts =
     filter_map both ~f:(function
       | First x -> Some x
       | Second _ -> None)
   in
-  let falses =
+  let seconds =
     filter_map both ~f:(function
       | First _ -> None
       | Second x -> Some x)
   in
-  trues, falses
+  firsts, seconds
 ;;
 
-let partition_tf t ~f = partitioni_tf t ~f:(fun _i x -> f x)
+let partitioni_tf t ~f =
+  partition_mapi t ~f:(fun i x -> if f i x then First x else Second x) [@nontail]
+;;
+
+let partition_map t ~f = partition_mapi t ~f:(fun _ x -> f x) [@nontail]
+let partition_tf t ~f = partitioni_tf t ~f:(fun _ x -> f x) [@nontail]
 let last t = t.(length t - 1)
 
 (* Convert to a sequence but does not attempt to protect against modification
    in the array. *)
 let to_sequence_mutable t =
   Sequence.unfold_step ~init:0 ~f:(fun i ->
-    if i >= length t then Sequence.Step.Done else Sequence.Step.Yield (t.(i), i + 1))
+    if i >= length t
+    then Sequence.Step.Done
+    else Sequence.Step.Yield { value = t.(i); state = i + 1 })
 ;;
 
 let to_sequence t = to_sequence_mutable (copy t)
@@ -721,7 +874,7 @@ let cartesian_product t1 t2 =
     let r = ref 0 in
     for i1 = 0 to n1 - 1 do
       for i2 = 0 to n2 - 1 do
-        t.(!r) <- (t1.(i1), t2.(i2));
+        t.(!r) <- t1.(i1), t2.(i2);
         incr r
       done
     done;
@@ -746,41 +899,31 @@ let transpose_exn tt =
 ;;
 
 include Binary_searchable.Make1 (struct
-    type nonrec 'a t = 'a t
+  type nonrec 'a t = 'a t
 
-    let get = get
-    let length = length
-  end)
+  let get = get
+  let length = length
+end)
 
 include Blit.Make1 (struct
-    type nonrec 'a t = 'a t
+  type nonrec 'a t = 'a t
 
-    let length = length
+  let length = length
 
-    let create_like ~len t =
-      if len = 0
-      then [||]
-      else (
-        assert (length t > 0);
-        create ~len t.(0))
-    ;;
+  let create_like ~len t =
+    if len = 0
+    then [||]
+    else (
+      assert (length t > 0);
+      create ~len t.(0))
+  ;;
 
-    let unsafe_blit = blit
-  end)
+  let unsafe_blit = unsafe_blit
+end)
 
 let invariant invariant_a t = iter t ~f:invariant_a
 
-(* Deprecated. [Obj.truncate] reduces the size of a block on the ocaml heap.  For arrays, the block
-   size is the array length. This holds even for float arrays. *)
-let unsafe_truncate t ~len =
-  if len <= 0 || len > length t
-  then
-    raise_s
-      (Sexp.message "Array.unsafe_truncate got invalid len" [ "len", sexp_of_int len ]);
-  if len < length t
-  then (Caml.Obj.truncate [@ocaml.alert "-deprecated"]) (Caml.Obj.repr t) len
-;;
-
 module Private = struct
   module Sort = Sort
+  module Sorter = Sorter
 end
