@@ -8,22 +8,17 @@ open! Import
     Furthermore, [blit] raises if [src_pos], [len], and [dst_pos] don't specify valid
     slices of [src] and [dst]. *)
 type ('src, 'dst) blit =
-  src:('src[@local])
-  -> src_pos:int
-  -> dst:('dst[@local])
-  -> dst_pos:int
-  -> len:int
-  -> unit
+  src:'src -> src_pos:int -> dst:'dst -> dst_pos:int -> len:int -> unit
 
 (** [blito] is like [blit], except that the [src_pos], [src_len], and [dst_pos] are
     optional (hence the "o" in "blito").  Also, we use [src_len] rather than [len] as a
     reminder that if [src_len] isn't supplied, then the default is to take the slice
     running from [src_pos] to the end of [src]. *)
 type ('src, 'dst) blito =
-  src:('src[@local])
+  src:'src
   -> ?src_pos:int (** default is [0] *)
   -> ?src_len:int (** default is [length src - src_pos] *)
-  -> dst:('dst[@local])
+  -> dst:'dst
   -> ?dst_pos:int (** default is [0] *)
   -> unit
   -> unit
@@ -99,7 +94,7 @@ end
 module type Sequence = sig
   type t
 
-  val length : (t[@local]) -> int
+  val length : t -> int
 end
 
 type 'a poly = 'a
@@ -111,7 +106,7 @@ module type Sequence1 = sig
       0]. *)
   val create_like : len:int -> 'a t -> 'a t
 
-  val length : (_ t[@local]) -> int
+  val length : _ t -> int
   val unsafe_blit : ('a t, 'a t) blit
 end
 
@@ -148,25 +143,25 @@ module type Blit = sig
 
   (** [Make] is for blitting between two values of the same monomorphic type. *)
   module Make (Sequence : sig
-      include Sequence
+    include Sequence
 
-      val create : len:int -> t
-      val unsafe_blit : (t, t) blit
-    end) : S with type t := Sequence.t
+    val create : len:int -> t
+    val unsafe_blit : (t, t) blit
+  end) : S with type t := Sequence.t
 
   (** [Make_distinct] is for blitting between values of distinct monomorphic types. *)
   module Make_distinct
-      (Src : Sequence) (Dst : sig
-                          include Sequence
+    (Src : Sequence) (Dst : sig
+      include Sequence
 
-                          val create : len:int -> t
-                          val unsafe_blit : (Src.t, t) blit
-                        end) : S_distinct with type src := Src.t with type dst := Dst.t
+      val create : len:int -> t
+      val unsafe_blit : (Src.t, t) blit
+    end) : S_distinct with type src := Src.t with type dst := Dst.t
 
   module Make_to_string (T : sig
-      type t
-    end)
-      (To_bytes : S_distinct with type src := T.t with type dst := bytes) :
+    type t
+  end)
+  (To_bytes : S_distinct with type src := T.t with type dst := bytes) :
     S_to_string with type t := T.t
 
   (** [Make1] is for blitting between two values of the same polymorphic type. *)
