@@ -3,17 +3,21 @@
 
 open! Import
 
-type t = bool [@@deriving_inline enumerate, sexp, sexp_grammar]
+type t = bool [@@deriving_inline enumerate, globalize, sexp, sexp_grammar]
 
-val all : t list
+include Ppx_enumerate_lib.Enumerable.S with type t := t
 
-include Ppx_sexp_conv_lib.Sexpable.S with type t := t
+val globalize : t -> t
 
-val t_sexp_grammar : Ppx_sexp_conv_lib.Sexp.Private.Raw_grammar.t
+include Sexplib0.Sexpable.S with type t := t
+
+val t_sexp_grammar : t Sexplib0.Sexp_grammar.t
 
 [@@@end]
 
 include Identifiable.S with type t := t
+include Ppx_compare_lib.Comparable.S_local with type t := t
+include Ppx_compare_lib.Equal.S_local with type t := t
 include Invariant.S with type t := t
 
 (**
@@ -21,6 +25,14 @@ include Invariant.S with type t := t
    - [to_int false = 0] *)
 val to_int : t -> int
 
+external select
+  :  bool
+  -> ('a[@local_opt])
+  -> ('a[@local_opt])
+  -> ('a[@local_opt])
+  = "caml_csel_value"
+  [@@noalloc] [@@no_effects] [@@no_coeffects] [@@builtin]
+
 module Non_short_circuiting : sig
   (** Non-short circuiting and branch-free boolean operators.
 
