@@ -15,6 +15,10 @@ let () =
     ~name:"discover"
     ~args:[ "-o", Set_string output, "FILENAME output file" ]
     (fun c ->
-      let has_popcnt = c_test c ~c_flags:[ "-mpopcnt" ] program in
+      let has_popcnt =
+        match ocaml_config_var_exn c "system" with
+        | "macosx" -> false
+        | _ -> c_test c ~c_flags:[ "-mpopcnt" ] program
+      in
       Flags.write_sexp !output (if has_popcnt then [ "-mpopcnt" ] else []))
 ;;
