@@ -17,26 +17,27 @@ module type Focused = sig
 
   val value : ('a, _) t -> default:'a -> 'a
   val to_option : ('a, _) t -> 'a option
-  val with_return : ((('a With_return.return[@local]) -> 'b)[@local]) -> ('a, 'b) t
+  val with_return : ('a With_return.return -> 'b) -> ('a, 'b) t
 
   val combine
     :  ('a, 'd) t
     -> ('b, 'd) t
-    -> f:(('a -> 'b -> 'c)[@local])
-    -> other:(('d -> 'd -> 'd)[@local])
+    -> f:('a -> 'b -> 'c)
+    -> other:('d -> 'd -> 'd)
     -> ('c, 'd) t
 
-  val combine_all : ('a, 'b) t list -> f:(('b -> 'b -> 'b)[@local]) -> ('a list, 'b) t
-  val combine_all_unit : (unit, 'b) t list -> f:(('b -> 'b -> 'b)[@local]) -> (unit, 'b) t
+  val combine_all : ('a, 'b) t list -> f:('b -> 'b -> 'b) -> ('a list, 'b) t
+  val combine_all_unit : (unit, 'b) t list -> f:('b -> 'b -> 'b) -> (unit, 'b) t
 end
 
 module type Either = sig
   type ('f, 's) t = ('f, 's) Either0.t =
     | First of 'f
     | Second of 's
-  [@@deriving_inline compare, hash, sexp, sexp_grammar]
+  [@@deriving_inline compare ~localize, hash, sexp, sexp_grammar]
 
   include Ppx_compare_lib.Comparable.S2 with type ('f, 's) t := ('f, 's) t
+  include Ppx_compare_lib.Comparable.S_local2 with type ('f, 's) t := ('f, 's) t
   include Ppx_hash_lib.Hashable.S2 with type ('f, 's) t := ('f, 's) t
   include Sexplib0.Sexpable.S2 with type ('f, 's) t := ('f, 's) t
 
@@ -51,27 +52,18 @@ module type Either = sig
 
   val swap : ('f, 's) t -> ('s, 'f) t
   val value : ('a, 'a) t -> 'a
-
-  val iter
-    :  ('a, 'b) t
-    -> first:(('a -> unit)[@local])
-    -> second:(('b -> unit)[@local])
-    -> unit
-
-  val value_map
-    :  ('a, 'b) t
-    -> first:(('a -> 'c)[@local])
-    -> second:(('b -> 'c)[@local])
-    -> 'c
-
-  val map
-    :  ('a, 'b) t
-    -> first:(('a -> 'c)[@local])
-    -> second:(('b -> 'd)[@local])
-    -> ('c, 'd) t
-
+  val iter : ('a, 'b) t -> first:('a -> unit) -> second:('b -> unit) -> unit
+  val value_map : ('a, 'b) t -> first:('a -> 'c) -> second:('b -> 'c) -> 'c
+  val map : ('a, 'b) t -> first:('a -> 'c) -> second:('b -> 'd) -> ('c, 'd) t
   val equal : ('f -> 'f -> bool) -> ('s -> 's -> bool) -> ('f, 's) t -> ('f, 's) t -> bool
 
+  val local_equal
+    :  ('f -> 'f -> bool)
+    -> ('s -> 's -> bool)
+    -> ('f, 's) t
+    -> ('f, 's) t
+    -> bool
+
   module type Focused = Focused
 
   module First : Focused with type ('a, 'b) t = ('a, 'b) t
