@@ -5,14 +5,15 @@
 open! Import
 include Info
 
-let raise t = raise (to_exn t)
-let raise_s sexp = raise (create_s sexp)
+let t_sexp_grammar : t Sexplib0.Sexp_grammar.t = { untyped = Any "Error.t" }
+let[@cold] raise t = raise (to_exn t)
+let[@cold] raise_s sexp = raise (create_s sexp)
 let to_info t = t
 let of_info t = t
 
 include Pretty_printer.Register_pp (struct
-    type nonrec t = t
+  type nonrec t = t
 
-    let module_name = "Base.Error"
-    let pp = pp
-  end)
+  let module_name = "Base.Error"
+  let pp = pp
+end)
