@@ -6,16 +6,24 @@ include Float0
 let raise_s = Error.raise_s
 
 module T = struct
-  type t = float [@@deriving_inline hash, sexp]
-  let (hash_fold_t :
-         Ppx_hash_lib.Std.Hash.state -> t -> Ppx_hash_lib.Std.Hash.state) =
+  type t = float [@@deriving_inline hash, globalize, sexp, sexp_grammar]
+
+  let (hash_fold_t : Ppx_hash_lib.Std.Hash.state -> t -> Ppx_hash_lib.Std.Hash.state) =
     hash_fold_float
+
   and (hash : t -> Ppx_hash_lib.Std.Hash.hash_value) =
-    let func = hash_float in fun x -> func x
-  let t_of_sexp = (float_of_sexp : Ppx_sexp_conv_lib.Sexp.t -> t)
-  let sexp_of_t = (sexp_of_float : t -> Ppx_sexp_conv_lib.Sexp.t)
+    let func = hash_float in
+    fun x -> func x
+  ;;
+
+  let (globalize : t -> t) = (globalize_float : t -> t)
+  let t_of_sexp = (float_of_sexp : Sexplib0.Sexp.t -> t)
+  let sexp_of_t = (sexp_of_float : t -> Sexplib0.Sexp.t)
+  let (t_sexp_grammar : t Sexplib0.Sexp_grammar.t) = float_sexp_grammar
+
   [@@@end]
 
+  let hashable : t Hashable.t = { hash; compare; sexp_of_t }
   let compare = Float_replace_polymorphic_compare.compare
 end
 
@@ -27,6 +35,7 @@ include Comparator.Make (T)
    functions are available within this module. *)
 open Float_replace_polymorphic_compare
 
+let invariant (_ : t) = ()
 let to_float x = x
 let of_float x = x
 
@@ -35,6 +44,8 @@ let of_string s =
   | _ -> invalid_argf "Float.of_string %s" s ()
 ;;
 
+let of_string_opt = float_of_string_opt
+
 external format_float : string -> float -> string = "caml_format_float"
 
 (* Stolen from [pervasives.ml].  Adds a "." at the end if needed.  It is in
@@ -179,8 +190,8 @@ let euler = 0x0.93C467E37DB0C7A4D1BE3F810152CB
 let of_int = Int.to_float
 let to_int = Int.of_float
 let of_int63 i = Int63.to_float i
-let of_int64 i = Caml.Int64.to_float i
-let to_int64 = Caml.Int64.of_float
+let of_int64 i = Stdlib.Int64.to_float i
+let to_int64 = Stdlib.Int64.of_float
 let iround_lbound = lower_bound_for_int Int.num_bits
 let iround_ubound = upper_bound_for_int Int.num_bits
 
@@ -280,7 +291,7 @@ let round_nearest_ub = 2. ** 52.
    and it gets rounded up to [1.] due to the round-ties-to-even rule. *)
 let one_ulp_less_than_half = one_ulp `Down 0.5
 
-let add_half_for_round_nearest t =
+let[@ocaml.inline always] add_half_for_round_nearest t =
   t
   +.
   if t = one_ulp_less_than_half
@@ -371,11 +382,8 @@ let iround ?(dir = `Nearest) t =
   | _ -> None
 ;;
 
-let is_inf x =
-  match classify_float x with
-  | FP_infinite -> true
-  | _ -> false
-;;
+let is_inf t = 1. /. t = 0.
+let is_finite t = t -. t = 0.
 
 let min_inan (x : t) y =
   if is_nan y then x else if is_nan x then y else if x < y then x else y
@@ -412,12 +420,14 @@ let round_up = ceil
 let round_towards_zero t = if t >= 0. then round_down t else round_up t
 
 (* see the comment above [round_nearest_lb] and [round_nearest_ub] for an explanation *)
-let round_nearest t =
+let[@ocaml.inline] round_nearest_inline t =
   if t > round_nearest_lb && t < round_nearest_ub
   then floor (add_half_for_round_nearest t)
   else t +. 0.
 ;;
 
+let round_nearest t = (round_nearest_inline [@ocaml.inlined always]) t
+
 let round_nearest_half_to_even t =
   if t <= round_nearest_lb || t >= round_nearest_ub
   then t +. 0.
@@ -432,7 +442,7 @@ let round_nearest_half_to_even t =
     else if diff_floor > diff_ceil
     then ceil_or_succ
     else if (* exact tie, pick the even *)
-      mod_float floor 2. = 0.
+            mod_float floor 2. = 0.
     then floor
     else ceil_or_succ)
 ;;
@@ -482,7 +492,7 @@ let int63_round_down_exn t =
 ;;
 
 let int63_round_nearest_portable_alloc_exn t0 =
-  let t = round_nearest t0 in
+  let t = (round_nearest_inline [@ocaml.inlined always]) t0 in
   if t > 0.
   then
     if t <= int63_round_ubound
@@ -501,7 +511,9 @@ let int63_round_nearest_portable_alloc_exn t0 =
       ()
 ;;
 
-let int63_round_nearest_arch64_noalloc_exn f = Int63.of_int (iround_nearest_exn f)
+let[@inline] int63_round_nearest_arch64_noalloc_exn f =
+  Int63.of_int (iround_nearest_exn f)
+;;
 
 let int63_round_nearest_exn =
   match Word_size.word_size with
@@ -524,46 +536,64 @@ module Class = struct
     | Normal
     | Subnormal
     | Zero
-  [@@deriving_inline compare, enumerate, sexp]
-  let compare = (Ppx_compare_lib.polymorphic_compare : t -> t -> int)
-  let all = ([Infinite; Nan; Normal; Subnormal; Zero] : t list)
+  [@@deriving_inline compare ~localize, enumerate, sexp, sexp_grammar]
+
+  let compare__local = (Stdlib.compare : t -> t -> int)
+  let compare = (fun a b -> compare__local a b : t -> t -> int)
+  let all = ([ Infinite; Nan; Normal; Subnormal; Zero ] : t list)
+
   let t_of_sexp =
-    (let _tp_loc = "src/float.ml.Class.t" in
+    (let error_source__007_ = "float.ml.Class.t" in
      function
-     | Ppx_sexp_conv_lib.Sexp.Atom ("infinite"|"Infinite") -> Infinite
-     | Ppx_sexp_conv_lib.Sexp.Atom ("nan"|"Nan") -> Nan
-     | Ppx_sexp_conv_lib.Sexp.Atom ("normal"|"Normal") -> Normal
-     | Ppx_sexp_conv_lib.Sexp.Atom ("subnormal"|"Subnormal") -> Subnormal
-     | Ppx_sexp_conv_lib.Sexp.Atom ("zero"|"Zero") -> Zero
-     | Ppx_sexp_conv_lib.Sexp.List ((Ppx_sexp_conv_lib.Sexp.Atom
-                                       ("infinite"|"Infinite"))::_) as sexp ->
-       Ppx_sexp_conv_lib.Conv_error.stag_no_args _tp_loc sexp
-     | Ppx_sexp_conv_lib.Sexp.List ((Ppx_sexp_conv_lib.Sexp.Atom
-                                       ("nan"|"Nan"))::_) as sexp ->
-       Ppx_sexp_conv_lib.Conv_error.stag_no_args _tp_loc sexp
-     | Ppx_sexp_conv_lib.Sexp.List ((Ppx_sexp_conv_lib.Sexp.Atom
-                                       ("normal"|"Normal"))::_) as sexp ->
-       Ppx_sexp_conv_lib.Conv_error.stag_no_args _tp_loc sexp
-     | Ppx_sexp_conv_lib.Sexp.List ((Ppx_sexp_conv_lib.Sexp.Atom
-                                       ("subnormal"|"Subnormal"))::_) as sexp ->
-       Ppx_sexp_conv_lib.Conv_error.stag_no_args _tp_loc sexp
-     | Ppx_sexp_conv_lib.Sexp.List ((Ppx_sexp_conv_lib.Sexp.Atom
-                                       ("zero"|"Zero"))::_) as sexp ->
-       Ppx_sexp_conv_lib.Conv_error.stag_no_args _tp_loc sexp
-     | Ppx_sexp_conv_lib.Sexp.List ((Ppx_sexp_conv_lib.Sexp.List _)::_) as sexp
-       -> Ppx_sexp_conv_lib.Conv_error.nested_list_invalid_sum _tp_loc sexp
-     | Ppx_sexp_conv_lib.Sexp.List [] as sexp ->
-       Ppx_sexp_conv_lib.Conv_error.empty_list_invalid_sum _tp_loc sexp
-     | sexp -> Ppx_sexp_conv_lib.Conv_error.unexpected_stag _tp_loc sexp :
-                 Ppx_sexp_conv_lib.Sexp.t -> t)
+     | Sexplib0.Sexp.Atom ("infinite" | "Infinite") -> Infinite
+     | Sexplib0.Sexp.Atom ("nan" | "Nan") -> Nan
+     | Sexplib0.Sexp.Atom ("normal" | "Normal") -> Normal
+     | Sexplib0.Sexp.Atom ("subnormal" | "Subnormal") -> Subnormal
+     | Sexplib0.Sexp.Atom ("zero" | "Zero") -> Zero
+     | Sexplib0.Sexp.List (Sexplib0.Sexp.Atom ("infinite" | "Infinite") :: _) as
+       sexp__008_ -> Sexplib0.Sexp_conv_error.stag_no_args error_source__007_ sexp__008_
+     | Sexplib0.Sexp.List (Sexplib0.Sexp.Atom ("nan" | "Nan") :: _) as sexp__008_ ->
+       Sexplib0.Sexp_conv_error.stag_no_args error_source__007_ sexp__008_
+     | Sexplib0.Sexp.List (Sexplib0.Sexp.Atom ("normal" | "Normal") :: _) as sexp__008_ ->
+       Sexplib0.Sexp_conv_error.stag_no_args error_source__007_ sexp__008_
+     | Sexplib0.Sexp.List (Sexplib0.Sexp.Atom ("subnormal" | "Subnormal") :: _) as
+       sexp__008_ -> Sexplib0.Sexp_conv_error.stag_no_args error_source__007_ sexp__008_
+     | Sexplib0.Sexp.List (Sexplib0.Sexp.Atom ("zero" | "Zero") :: _) as sexp__008_ ->
+       Sexplib0.Sexp_conv_error.stag_no_args error_source__007_ sexp__008_
+     | Sexplib0.Sexp.List (Sexplib0.Sexp.List _ :: _) as sexp__006_ ->
+       Sexplib0.Sexp_conv_error.nested_list_invalid_sum error_source__007_ sexp__006_
+     | Sexplib0.Sexp.List [] as sexp__006_ ->
+       Sexplib0.Sexp_conv_error.empty_list_invalid_sum error_source__007_ sexp__006_
+     | sexp__006_ ->
+       Sexplib0.Sexp_conv_error.unexpected_stag error_source__007_ sexp__006_
+      : Sexplib0.Sexp.t -> t)
+  ;;
+
   let sexp_of_t =
     (function
-      | Infinite -> Ppx_sexp_conv_lib.Sexp.Atom "Infinite"
-      | Nan -> Ppx_sexp_conv_lib.Sexp.Atom "Nan"
-      | Normal -> Ppx_sexp_conv_lib.Sexp.Atom "Normal"
-      | Subnormal -> Ppx_sexp_conv_lib.Sexp.Atom "Subnormal"
-      | Zero -> Ppx_sexp_conv_lib.Sexp.Atom "Zero" : t ->
-        Ppx_sexp_conv_lib.Sexp.t)
+     | Infinite -> Sexplib0.Sexp.Atom "Infinite"
+     | Nan -> Sexplib0.Sexp.Atom "Nan"
+     | Normal -> Sexplib0.Sexp.Atom "Normal"
+     | Subnormal -> Sexplib0.Sexp.Atom "Subnormal"
+     | Zero -> Sexplib0.Sexp.Atom "Zero"
+      : t -> Sexplib0.Sexp.t)
+  ;;
+
+  let (t_sexp_grammar : t Sexplib0.Sexp_grammar.t) =
+    { untyped =
+        Variant
+          { case_sensitivity = Case_sensitive_except_first_character
+          ; clauses =
+              [ No_tag { name = "Infinite"; clause_kind = Atom_clause }
+              ; No_tag { name = "Nan"; clause_kind = Atom_clause }
+              ; No_tag { name = "Normal"; clause_kind = Atom_clause }
+              ; No_tag { name = "Subnormal"; clause_kind = Atom_clause }
+              ; No_tag { name = "Zero"; clause_kind = Atom_clause }
+              ]
+          }
+    }
+  ;;
+
   [@@@end]
 
   let to_string t = string_of_sexp (sexp_of_t t)
@@ -580,13 +610,11 @@ let classify t =
   | FP_nan -> C.Nan
 ;;
 
-let is_finite t = not (t = infinity || t = neg_infinity || is_nan t)
-
 let insert_underscores ?(delimiter = '_') ?(strip_zero = false) string =
   match String.lsplit2 string ~on:'.' with
-  | None -> Int_conversions.insert_delimiter string ~delimiter
+  | None -> Int_string_conversions.insert_delimiter string ~delimiter
   | Some (left, right) ->
-    let left = Int_conversions.insert_delimiter left ~delimiter in
+    let left = Int_string_conversions.insert_delimiter left ~delimiter in
     let right =
       if strip_zero then String.rstrip right ~drop:(fun c -> Char.( = ) c '0') else right
     in
@@ -595,14 +623,17 @@ let insert_underscores ?(delimiter = '_') ?(strip_zero = false) string =
      | _ -> left ^ "." ^ right)
 ;;
 
-let to_string_hum ?delimiter ?(decimals = 3) ?strip_zero f =
+let to_string_hum ?delimiter ?(decimals = 3) ?strip_zero ?(explicit_plus = false) f =
   if Int_replace_polymorphic_compare.( < ) decimals 0
   then invalid_argf "to_string_hum: invalid argument ~decimals=%d" decimals ();
   match classify f with
   | Class.Infinite -> if f > 0. then "inf" else "-inf"
   | Class.Nan -> "nan"
   | Class.Normal | Class.Subnormal | Class.Zero ->
-    insert_underscores (sprintf "%.*f" decimals f) ?delimiter ?strip_zero
+    let s =
+      if explicit_plus then sprintf "%+.*f" decimals f else sprintf "%.*f" decimals f
+    in
+    insert_underscores s ?delimiter ?strip_zero
 ;;
 
 let sexp_of_t t =
@@ -646,7 +677,7 @@ let to_padded_compact_string_custom t ?(prefix = "") ~kilo ~mega ~giga ~tera ?pe
     else if diff_right > diff_left
     then k
     else if (* a tie *)
-      Int_replace_polymorphic_compare.( = ) (k mod 2) 0
+            Int_replace_polymorphic_compare.( = ) (k mod 2) 0
     then k
     else k + 1
   in
@@ -793,7 +824,7 @@ let round_gen x ~how =
          integers are, but their inverses in most cases are not. *)
       let abs_dd = Int.abs dd in
       if abs_dd > 22 || sd >= 16
-      (* 10**22 is exactly representable as a float, but 10**23 is not, so use the slow
+         (* 10**22 is exactly representable as a float, but 10**23 is not, so use the slow
          path.  Similarly, if we need 16 significant digits in the result, then the integer
          [round_nearest (x <op> order)] might not be exactly representable as a float, since
          for some ranges we only have 15 digits of precision guaranteed.
@@ -819,10 +850,10 @@ let round_gen x ~how =
 let round_significant x ~significant_digits =
   if Int_replace_polymorphic_compare.( <= ) significant_digits 0
   then
-    raise
-      (Invalid_argument
-         ("Float.round_significant: invalid argument significant_digits:"
-          ^ Int.to_string significant_digits))
+    invalid_argf
+      "Float.round_significant: invalid argument significant_digits:%d"
+      significant_digits
+      ()
   else round_gen x ~how:(`significant_digits significant_digits)
 ;;
 
@@ -833,13 +864,21 @@ let clamp_exn t ~min ~max =
   (* Also fails if [min] or [max] is nan *)
   assert (min <= max);
   (* clamp_unchecked is in float0.ml *)
-  clamp_unchecked t ~min ~max
+  clamp_unchecked
+    ~to_clamp_maybe_nan:t
+    ~min_which_is_not_nan:min
+    ~max_which_is_not_nan:max
 ;;
 
 let clamp t ~min ~max =
   (* Also fails if [min] or [max] is nan *)
   if min <= max
-  then Ok (clamp_unchecked t ~min ~max)
+  then
+    Ok
+      (clamp_unchecked
+         ~to_clamp_maybe_nan:t
+         ~min_which_is_not_nan:min
+         ~max_which_is_not_nan:max)
   else
     Or_error.error_s
       (Sexp.message
@@ -852,9 +891,10 @@ let ( - ) = ( -. )
 let ( * ) = ( *. )
 let ( ** ) = ( ** )
 let ( / ) = ( /. )
+let ( % ) = ( %. )
 let ( ~- ) = ( ~-. )
 
-let sign_exn t : Sign.t =
+let[@inline] sign_exn t : Sign.t =
   if t > 0.
   then Pos
   else if t < 0.
@@ -869,8 +909,8 @@ let sign_or_nan t : Sign_or_nan.t =
 ;;
 
 let ieee_negative t =
-  let bits = Caml.Int64.bits_of_float t in
-  Poly.(bits < Caml.Int64.zero)
+  let bits = Stdlib.Int64.bits_of_float t in
+  Poly.(bits < Stdlib.Int64.zero)
 ;;
 
 let exponent_bits = 11
@@ -881,14 +921,15 @@ let mantissa_mask = Int63.(shift_left one mantissa_bits - one)
 let mantissa_mask64 = Int63.to_int64 mantissa_mask
 
 let ieee_exponent t =
-  let bits = Caml.Int64.bits_of_float t in
+  let bits = Stdlib.Int64.bits_of_float t in
   Int64.(bit_and (shift_right_logical bits mantissa_bits) exponent_mask64)
-  |> Caml.Int64.to_int
+  |> Stdlib.Int64.to_int
 ;;
 
 let ieee_mantissa t =
-  let bits = Caml.Int64.bits_of_float t in
-  Int63.of_int64_exn Caml.Int64.(logand bits mantissa_mask64)
+  let bits = Stdlib.Int64.bits_of_float t in
+  (* This is safe because mantissa_mask64 < Int63.max_value *)
+  (Int63.of_int64_trunc [@inlined]) Stdlib.Int64.(logand bits mantissa_mask64)
 ;;
 
 let create_ieee_exn ~negative ~exponent ~mantissa =
@@ -902,11 +943,13 @@ let create_ieee_exn ~negative ~exponent ~mantissa =
       (Int63.to_string mantissa_mask)
       ()
   else (
-    let sign_bits = if negative then Caml.Int64.min_int else Caml.Int64.zero in
-    let expt_bits = Caml.Int64.shift_left (Caml.Int64.of_int exponent) mantissa_bits in
+    let sign_bits = if negative then Stdlib.Int64.min_int else Stdlib.Int64.zero in
+    let expt_bits =
+      Stdlib.Int64.shift_left (Stdlib.Int64.of_int exponent) mantissa_bits
+    in
     let mant_bits = Int63.to_int64 mantissa in
-    let bits = Caml.Int64.(logor sign_bits (logor expt_bits mant_bits)) in
-    Caml.Int64.float_of_bits bits)
+    let bits = Stdlib.Int64.(logor sign_bits (logor expt_bits mant_bits)) in
+    Stdlib.Int64.float_of_bits bits)
 ;;
 
 let create_ieee ~negative ~exponent ~mantissa =
@@ -920,42 +963,14 @@ module Terse = struct
   let to_string x = Printf.sprintf "%.8G" x
   let sexp_of_t x = Sexp.Atom (to_string x)
   let of_string x = of_string x
+  let t_sexp_grammar = t_sexp_grammar
 end
 
-let validate_ordinary t =
-  Validate.of_error_opt
-    (let module C = Class in
-     match classify t with
-     | C.Normal | C.Subnormal | C.Zero -> None
-     | C.Infinite -> Some "value is infinite"
-     | C.Nan -> Some "value is NaN")
-;;
-
-module V = struct
-  module ZZ = Comparable.Validate (T)
-
-  let validate_bound ~min ~max t =
-    Validate.first_failure (validate_ordinary t) (ZZ.validate_bound t ~min ~max)
-  ;;
-
-  let validate_lbound ~min t =
-    Validate.first_failure (validate_ordinary t) (ZZ.validate_lbound t ~min)
-  ;;
-
-  let validate_ubound ~max t =
-    Validate.first_failure (validate_ordinary t) (ZZ.validate_ubound t ~max)
-  ;;
-end
-
-include V
-
 include Comparable.With_zero (struct
-    include T
+  include T
 
-    let zero = zero
-
-    include V
-  end)
+  let zero = zero
+end)
 
 (* These are partly here as a performance hack to avoid some boxing we're getting with
    the versions we get from [With_zero].  They also make [Float.is_negative nan] and
@@ -967,17 +982,18 @@ let is_negative t = t < 0.
 let is_non_positive t = t <= 0.
 
 include Pretty_printer.Register (struct
-    include T
+  include T
 
-    let module_name = "Base.Float"
-    let to_string = to_string
-  end)
+  let module_name = "Base.Float"
+  let to_string = to_string
+end)
 
 module O = struct
   let ( + ) = ( + )
   let ( - ) = ( - )
   let ( * ) = ( * )
   let ( / ) = ( / )
+  let ( % ) = ( % )
   let ( ~- ) = ( ~- )
   let ( ** ) = ( ** )
 
@@ -995,6 +1011,7 @@ module O_dot = struct
   let ( +. ) = ( + )
   let ( -. ) = ( - )
   let ( /. ) = ( / )
+  let ( %. ) = ( % )
   let ( ~-. ) = ( ~- )
   let ( **. ) = ( ** )
 end
@@ -1017,6 +1034,34 @@ end
    this module. *)
 include Float_replace_polymorphic_compare
 
-(* These functions specifically replace defaults in replace_polymorphic_compare *)
-let min (x : t) y = if is_nan x || is_nan y then nan else if x < y then x else y
-let max (x : t) y = if is_nan x || is_nan y then nan else if x > y then x else y
+(* These functions specifically replace defaults in replace_polymorphic_compare.
+
+   The desired behavior here is to propagate a nan if either argument is nan. Because the
+   first comparison will always return false if either argument is nan, it suffices to
+   check if x is nan. Then, when x is nan or both x and y are nan, we return x = nan; and
+   when y is nan but not x, we return y = nan.
+
+   There are various ways to implement these functions.  The benchmark below shows a few
+   different versions.  This benchmark was run over an array of random floats (none of
+   which are nan).
+
+   ┌────────────────────────────────────────────────┬──────────┐
+   │ Name                                           │ Time/Run │
+   ├────────────────────────────────────────────────┼──────────┤
+   │ if is_nan x then x else if x < y then x else y │   2.42us │
+   │ if is_nan x || x < y then x else y             │   2.02us │
+   │ if x < y || is_nan x then x else y             │   1.88us │
+   └────────────────────────────────────────────────┴──────────┘
+
+   The benchmark below was run when x > y is always true (again, no nan values).
+
+   ┌────────────────────────────────────────────────┬──────────┐
+   │ Name                                           │ Time/Run │
+   ├────────────────────────────────────────────────┼──────────┤
+   │ if is_nan x then x else if x < y then x else y │   2.83us │
+   │ if is_nan x || x < y then x else y             │   1.97us │
+   │ if x < y || is_nan x then x else y             │   1.56us │
+   └────────────────────────────────────────────────┴──────────┘
+*)
+let min (x : t) y = if x < y || is_nan x then x else y
+let max (x : t) y = if x > y || is_nan x then x else y
