@@ -28,8 +28,8 @@ include Hash_intf
 (** Builtin folding-style hash functions, abstracted over [Hash_intf.S] *)
 module Folding (Hash : Hash_intf.S) :
   Hash_intf.Builtin_intf
-  with type state = Hash.state
-   and type hash_value = Hash.hash_value = struct
+    with type state = Hash.state
+     and type hash_value = Hash.hash_value = struct
   type state = Hash.state
   type hash_value = Hash.hash_value
   type 'a folder = state -> 'a -> state
@@ -43,7 +43,7 @@ module Folding (Hash : Hash_intf.S) :
 
   (* This ignores the sign bit on 32-bit architectures, but it's unlikely to lead to
      frequent collisions (min_value colliding with 0 is the most likely one).  *)
-  let hash_fold_int32 = as_int Caml.Int32.to_int
+  let hash_fold_int32 = as_int Stdlib.Int32.to_int
   let hash_fold_char = as_int Char.to_int
 
   let hash_fold_bool =
@@ -52,7 +52,7 @@ module Folding (Hash : Hash_intf.S) :
       | false -> 0)
   ;;
 
-  let hash_fold_nativeint s x = hash_fold_int64 s (Caml.Int64.of_nativeint x)
+  let hash_fold_nativeint s x = hash_fold_int64 s (Stdlib.Int64.of_nativeint x)
 
   let hash_fold_option hash_fold_elem s = function
     | None -> hash_fold_int s 0
@@ -75,7 +75,7 @@ module Folding (Hash : Hash_intf.S) :
     s
   ;;
 
-  let hash_fold_lazy_t hash_fold_elem s x = hash_fold_elem s (Caml.Lazy.force x)
+  let hash_fold_lazy_t hash_fold_elem s x = hash_fold_elem s (Stdlib.Lazy.force x)
   let hash_fold_ref_frozen hash_fold_elem s x = hash_fold_elem s !x
 
   let rec hash_fold_array_frozen_i hash_fold_elem s array i =
@@ -117,9 +117,9 @@ end
 
 module F (Hash : Hash_intf.S) :
   Hash_intf.Full
-  with type hash_value = Hash.hash_value
-   and type state = Hash.state
-   and type seed = Hash.seed = struct
+    with type hash_value = Hash.hash_value
+     and type state = Hash.state
+     and type seed = Hash.seed = struct
   include Hash
 
   type 'a folder = state -> 'a -> state
@@ -137,26 +137,34 @@ end
 module Internalhash : sig
   include
     Hash_intf.S
-    with type state = Base_internalhash_types.state
-     (* We give a concrete type for [state], albeit only partially exposed (see
+      with type state = Base_internalhash_types.state
+      (* We give a concrete type for [state], albeit only partially exposed (see
         Base_internalhash_types), so that it unifies with the same type in [Base_boot],
         and to allow optimizations for the immediate type. *)
-     and type seed = Base_internalhash_types.seed
-     and type hash_value = Base_internalhash_types.hash_value
+       and type seed = Base_internalhash_types.seed
+       and type hash_value = Base_internalhash_types.hash_value
 
-  external fold_int64 : state -> int64 -> state = "Base_internalhash_fold_int64"
-  [@@noalloc]
+  external fold_int64
+    :  state
+    -> (int64[@unboxed])
+    -> state
+    = "Base_internalhash_fold_int64" "Base_internalhash_fold_int64_unboxed"
+    [@@noalloc]
 
   external fold_int : state -> int -> state = "Base_internalhash_fold_int" [@@noalloc]
 
-  external fold_float : state -> float -> state = "Base_internalhash_fold_float"
-  [@@noalloc]
+  external fold_float
+    :  state
+    -> (float[@unboxed])
+    -> state
+    = "Base_internalhash_fold_float" "Base_internalhash_fold_float_unboxed"
+    [@@noalloc]
 
   external fold_string : state -> string -> state = "Base_internalhash_fold_string"
-  [@@noalloc]
+    [@@noalloc]
 
   external get_hash_value : state -> hash_value = "Base_internalhash_get_hash_value"
-  [@@noalloc]
+    [@@noalloc]
 end = struct
   let description = "internalhash"
 
@@ -182,17 +190,7 @@ module T = struct
 
   module Builtin = struct
     module Folding = Folding (Internalhash)
-
-    include (
-      Folding :
-        Hash_intf.Builtin_hash_fold_intf
-      with type state := state
-       and type 'a folder := 'a folder)
-
-    let hash_nativeint = Folding.hash_nativeint
-    let hash_int64 = Folding.hash_int64
-    let hash_int32 = Folding.hash_int32
-    let hash_string = Folding.hash_string
+    include Folding
 
     (* [Folding] provides some default implementations for the [hash_*] functions below,
        but they are inefficient for some use-cases because of the use of the [hash_fold]
@@ -224,7 +222,11 @@ module T = struct
 
     let hash_bool x = if x then 1 else 0
 
-    external hash_float : float -> int = "Base_hash_double" [@@noalloc]
+    external hash_float
+      :  (float[@unboxed])
+      -> int
+      = "Base_hash_double" "Base_hash_double_unboxed"
+      [@@noalloc]
 
     let hash_unit () = 0
   end
