@@ -23,10 +23,10 @@ module Accessors = struct
         match f elt with
         | None -> ()
         | Some _ as o -> r.return o);
-      None)
+      None) [@nontail]
   ;;
 
-  let find t ~f = find_map t ~f:(fun a -> if f a then Some a else None)
+  let find t ~f = find_map t ~f:(fun a -> if f a then Some a else None) [@nontail]
   let add t k = Hashtbl.set t ~key:k ~data:()
 
   let strict_add t k =
@@ -49,14 +49,18 @@ module Accessors = struct
   ;;
 
   let strict_remove_exn t k = Or_error.ok_exn (strict_remove t k)
-  let fold t ~init ~f = Hashtbl.fold t ~init ~f:(fun ~key ~data:() acc -> f acc key)
+
+  let fold t ~init ~f =
+    Hashtbl.fold t ~init ~f:(fun ~key ~data:() acc -> f acc key) [@nontail]
+  ;;
+
   let iter t ~f = Hashtbl.iter_keys t ~f
   let count t ~f = Container.count ~fold t ~f
   let sum m t ~f = Container.sum ~fold m t ~f
   let min_elt t ~compare = Container.min_elt ~fold t ~compare
   let max_elt t ~compare = Container.max_elt ~fold t ~compare
   let fold_result t ~init ~f = Container.fold_result ~fold ~init ~f t
-  let fold_until t ~init ~f = Container.fold_until ~fold ~init ~f t
+  let fold_until t ~init ~f ~finish = Container.fold_until ~fold ~init ~f t ~finish
   let to_list = Hashtbl.keys
 
   let sexp_of_t sexp_of_e t =
@@ -75,11 +79,11 @@ module Accessors = struct
         acc))
   ;;
 
-  let exists t ~f = Hashtbl.existsi t ~f:(fun ~key ~data:() -> f key)
+  let exists t ~f = Hashtbl.existsi t ~f:(fun ~key ~data:() -> f key) [@nontail]
   let for_all t ~f = not (Hashtbl.existsi t ~f:(fun ~key ~data:() -> not (f key)))
   let equal t1 t2 = Hashtbl.equal (fun () () -> true) t1 t2
   let copy t = Hashtbl.copy t
-  let filter t ~f = Hashtbl.filteri t ~f:(fun ~key ~data:() -> f key)
+  let filter t ~f = Hashtbl.filteri t ~f:(fun ~key ~data:() -> f key) [@nontail]
   let union t1 t2 = Hashtbl.merge t1 t2 ~f:(fun ~key:_ _ -> Some ())
   let diff t1 t2 = filter t1 ~f:(fun key -> not (Hashtbl.mem t2 key))
 
@@ -94,7 +98,7 @@ module Accessors = struct
   ;;
 
   let of_hashtbl_keys hashtbl = Hashtbl.map hashtbl ~f:ignore
-  let to_hashtbl t ~f = Hashtbl.mapi t ~f:(fun ~key ~data:() -> f key)
+  let to_hashtbl t ~f = Hashtbl.mapi t ~f:(fun ~key ~data:() -> f key) [@nontail]
 end
 
 include Accessors
@@ -126,23 +130,19 @@ let t_of_sexp m e_of_sexp sexp =
 ;;
 
 module Creators (Elt : sig
-    type 'a t
+  type 'a t
 
-    val hashable : 'a t Hashable.t
-  end) : sig
-  type 'a t_ = 'a Elt.t t
-
-  val t_of_sexp : (Sexp.t -> 'a Elt.t) -> Sexp.t -> 'a t_
+  val hashable : 'a t Hashable.t
+end) : sig
+  val t_of_sexp : (Sexp.t -> 'a Elt.t) -> Sexp.t -> 'a Elt.t t
 
   include
     Creators_generic
-    with type 'a t := 'a t_
-    with type 'a elt := 'a Elt.t
-    with type ('elt, 'z) create_options :=
-      ('elt, 'z) create_options_without_first_class_module
+      with type 'a t := 'a Elt.t t
+      with type 'a elt := 'a Elt.t
+      with type ('elt, 'z) create_options :=
+        ('elt, 'z) create_options_without_first_class_module
 end = struct
-  type 'a t_ = 'a Elt.t t
-
   let create ?growth_allowed ?size () =
     create ?growth_allowed ?size (Hashable.to_key Elt.hashable)
   ;;
@@ -161,38 +161,21 @@ module Poly = struct
   let hashable = poly_hashable
 
   include Creators (struct
-      type 'a t = 'a
+    type 'a t = 'a
 
-      let hashable = hashable
-    end)
+    let hashable = hashable
+  end)
 
   include Accessors
 
   let sexp_of_t = sexp_of_t
+  let t_sexp_grammar grammar = Sexplib0.Sexp_grammar.coerce (List.t_sexp_grammar grammar)
 end
 
 module M (Elt : T.T) = struct
   type nonrec t = Elt.t t
 end
 
-module type Sexp_of_m = sig
-  type t [@@deriving_inline sexp_of]
-
-  val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
-
-  [@@@end]
-end
-
-module type M_of_sexp = sig
-  type t [@@deriving_inline of_sexp]
-
-  val t_of_sexp : Ppx_sexp_conv_lib.Sexp.t -> t
-
-  [@@@end]
-
-  include Hashtbl_intf.Key.S with type t := t
-end
-
 let sexp_of_m__t (type elt) (module Elt : Sexp_of_m with type t = elt) t =
   sexp_of_t Elt.sexp_of_t t
 ;;
@@ -201,6 +184,12 @@ let m__t_of_sexp (type elt) (module Elt : M_of_sexp with type t = elt) sexp =
   t_of_sexp (module Elt) Elt.t_of_sexp sexp
 ;;
 
+let m__t_sexp_grammar (type elt) (module Elt : M_sexp_grammar with type t = elt) =
+  Sexplib0.Sexp_grammar.coerce (list_sexp_grammar Elt.t_sexp_grammar)
+;;
+
+let equal_m__t (module _ : Equal_m) t1 t2 = equal t1 t2
+
 module Private = struct
   let hashable = Hashtbl.Private.hashable
 end
