@@ -5,41 +5,41 @@
 include (
   Shadow_stdlib :
     module type of struct
-    include Shadow_stdlib
-  end
-  with type 'a ref := 'a ref
-  with type ('a, 'b, 'c) format := ('a, 'b, 'c) format
-  with type ('a, 'b, 'c, 'd) format4 := ('a, 'b, 'c, 'd) format4
-  with type ('a, 'b, 'c, 'd, 'e, 'f) format6 := ('a, 'b, 'c, 'd, 'e, 'f) format6
-  (* These modules are redefined in Base *)
-  with module Array := Shadow_stdlib.Array
-  with module Atomic := Shadow_stdlib.Atomic
-  with module Bool := Shadow_stdlib.Bool
-  with module Buffer := Shadow_stdlib.Buffer
-  with module Bytes := Shadow_stdlib.Bytes
-  with module Char := Shadow_stdlib.Char
-  with module Either := Shadow_stdlib.Either
-  with module Float := Shadow_stdlib.Float
-  with module Hashtbl := Shadow_stdlib.Hashtbl
-  with module Int := Shadow_stdlib.Int
-  with module Int32 := Shadow_stdlib.Int32
-  with module Int64 := Shadow_stdlib.Int64
-  with module Lazy := Shadow_stdlib.Lazy
-  with module List := Shadow_stdlib.List
-  with module Map := Shadow_stdlib.Map
-  with module Nativeint := Shadow_stdlib.Nativeint
-  with module Option := Shadow_stdlib.Option
-  with module Printf := Shadow_stdlib.Printf
-  with module Queue := Shadow_stdlib.Queue
-  with module Random := Shadow_stdlib.Random
-  with module Result := Shadow_stdlib.Result
-  with module Set := Shadow_stdlib.Set
-  with module Stack := Shadow_stdlib.Stack
-  with module String := Shadow_stdlib.String
-  with module Sys := Shadow_stdlib.Sys
-  with module Uchar := Shadow_stdlib.Uchar
-  with module Unit := Shadow_stdlib.Unit)
-  [@ocaml.warning "-3"]
+      include Shadow_stdlib
+    end
+    with type 'a ref := 'a ref
+    with type ('a, 'b, 'c) format := ('a, 'b, 'c) format
+    with type ('a, 'b, 'c, 'd) format4 := ('a, 'b, 'c, 'd) format4
+    with type ('a, 'b, 'c, 'd, 'e, 'f) format6 := ('a, 'b, 'c, 'd, 'e, 'f) format6
+    (* These modules are redefined in Base *)
+    with module Array := Shadow_stdlib.Array
+    with module Atomic := Shadow_stdlib.Atomic
+    with module Bool := Shadow_stdlib.Bool
+    with module Buffer := Shadow_stdlib.Buffer
+    with module Bytes := Shadow_stdlib.Bytes
+    with module Char := Shadow_stdlib.Char
+    with module Either := Shadow_stdlib.Either
+    with module Float := Shadow_stdlib.Float
+    with module Hashtbl := Shadow_stdlib.Hashtbl
+    with module Int := Shadow_stdlib.Int
+    with module Int32 := Shadow_stdlib.Int32
+    with module Int64 := Shadow_stdlib.Int64
+    with module Lazy := Shadow_stdlib.Lazy
+    with module List := Shadow_stdlib.List
+    with module Map := Shadow_stdlib.Map
+    with module Nativeint := Shadow_stdlib.Nativeint
+    with module Option := Shadow_stdlib.Option
+    with module Printf := Shadow_stdlib.Printf
+    with module Queue := Shadow_stdlib.Queue
+    with module Random := Shadow_stdlib.Random
+    with module Result := Shadow_stdlib.Result
+    with module Set := Shadow_stdlib.Set
+    with module Stack := Shadow_stdlib.Stack
+    with module String := Shadow_stdlib.String
+    with module Sys := Shadow_stdlib.Sys
+    with module Uchar := Shadow_stdlib.Uchar
+    with module Unit := Shadow_stdlib.Unit)
+[@ocaml.warning "-3"]
 
 type 'a ref = 'a Stdlib.ref = { mutable contents : 'a }
 
@@ -61,7 +61,7 @@ external not : (bool[@local_opt]) -> bool = "%boolnot"
    performance difference is noticeable. *)
 let bool_to_int (x : bool) : int = Stdlib.Obj.magic x
 
-(* This need to be declared as an external for the warnings to work properly *)
+(* This needs to be declared as an external for the warnings to work properly *)
 external ignore : _ -> unit = "%ignore"
 
 let ( != ) = Stdlib.( != )
@@ -87,7 +87,9 @@ module Int_replace_polymorphic_compare = struct
   external ( <= ) : (int[@local_opt]) -> (int[@local_opt]) -> bool = "%lessequal"
   external ( >= ) : (int[@local_opt]) -> (int[@local_opt]) -> bool = "%greaterequal"
   external compare : (int[@local_opt]) -> (int[@local_opt]) -> int = "%compare"
+  external compare__local : (int[@local_opt]) -> (int[@local_opt]) -> int = "%compare"
   external equal : (int[@local_opt]) -> (int[@local_opt]) -> bool = "%equal"
+  external equal__local : (int[@local_opt]) -> (int[@local_opt]) -> bool = "%equal"
 
   let ascending (x : int) y = compare x y
   let descending (x : int) y = compare y x
@@ -107,7 +109,9 @@ module Int32_replace_polymorphic_compare = struct
   let ascending (x : Stdlib.Int32.t) y = Poly.ascending x y
   let descending (x : Stdlib.Int32.t) y = Poly.descending x y
   let compare (x : Stdlib.Int32.t) y = Poly.compare x y
+  let compare__local (x : Stdlib.Int32.t) y = Poly.compare x y
   let equal (x : Stdlib.Int32.t) y = Poly.equal x y
+  let equal__local (x : Stdlib.Int32.t) y = Poly.equal x y
   let max (x : Stdlib.Int32.t) y = Bool0.select (x >= y) x y
   let min (x : Stdlib.Int32.t) y = Bool0.select (x <= y) x y
 end
@@ -157,12 +161,24 @@ module Int64_replace_polymorphic_compare = struct
     -> int
     = "%compare"
 
+  external compare__local
+    :  (Stdlib.Int64.t[@local_opt])
+    -> (Stdlib.Int64.t[@local_opt])
+    -> int
+    = "%compare"
+
   external equal
     :  (Stdlib.Int64.t[@local_opt])
     -> (Stdlib.Int64.t[@local_opt])
     -> bool
     = "%equal"
 
+  external equal__local
+    :  (Stdlib.Int64.t[@local_opt])
+    -> (Stdlib.Int64.t[@local_opt])
+    -> bool
+    = "%equal"
+
   let ascending (x : Stdlib.Int64.t) y = Poly.ascending x y
   let descending (x : Stdlib.Int64.t) y = Poly.descending x y
   let max (x : Stdlib.Int64.t) y = Bool0.select (x >= y) x y
@@ -179,7 +195,9 @@ module Nativeint_replace_polymorphic_compare = struct
   let ascending (x : Stdlib.Nativeint.t) y = Poly.ascending x y
   let descending (x : Stdlib.Nativeint.t) y = Poly.descending x y
   let compare (x : Stdlib.Nativeint.t) y = Poly.compare x y
+  let compare__local (x : Stdlib.Nativeint.t) y = Poly.compare x y
   let equal (x : Stdlib.Nativeint.t) y = Poly.equal x y
+  let equal__local (x : Stdlib.Nativeint.t) y = Poly.equal x y
   let max (x : Stdlib.Nativeint.t) y = Bool0.select (x >= y) x y
   let min (x : Stdlib.Nativeint.t) y = Bool0.select (x <= y) x y
 end
@@ -194,7 +212,9 @@ module Bool_replace_polymorphic_compare = struct
   let ascending (x : bool) y = Poly.ascending x y
   let descending (x : bool) y = Poly.descending x y
   let compare (x : bool) y = Poly.compare x y
+  let compare__local (x : bool) y = Poly.compare x y
   let equal (x : bool) y = Poly.equal x y
+  let equal__local (x : bool) y = Poly.equal x y
   let max (x : bool) y = Bool0.select (x >= y) x y
   let min (x : bool) y = Bool0.select (x <= y) x y
 end
@@ -209,13 +229,18 @@ module Char_replace_polymorphic_compare = struct
   let ascending (x : char) y = Poly.ascending x y
   let descending (x : char) y = Poly.descending x y
   let compare (x : char) y = Poly.compare x y
+  let compare__local (x : char) y = Poly.compare x y
   let equal (x : char) y = Poly.equal x y
+  let equal__local (x : char) y = Poly.equal x y
   let max (x : char) y = Bool0.select (x >= y) x y
   let min (x : char) y = Bool0.select (x <= y) x y
 end
 
 module Uchar_replace_polymorphic_compare = struct
-  let i x = Stdlib.Uchar.to_int x
+  open struct
+    external i : (Stdlib.Uchar.t[@local_opt]) -> int = "%identity"
+  end
+
   let ( < ) (x : Stdlib.Uchar.t) y = Int_replace_polymorphic_compare.( < ) (i x) (i y)
   let ( <= ) (x : Stdlib.Uchar.t) y = Int_replace_polymorphic_compare.( <= ) (i x) (i y)
   let ( <> ) (x : Stdlib.Uchar.t) y = Int_replace_polymorphic_compare.( <> ) (i x) (i y)
@@ -233,21 +258,33 @@ module Uchar_replace_polymorphic_compare = struct
 
   let compare (x : Stdlib.Uchar.t) y = Int_replace_polymorphic_compare.compare (i x) (i y)
   let equal (x : Stdlib.Uchar.t) y = Int_replace_polymorphic_compare.equal (i x) (i y)
+
+  let compare__local (x : Stdlib.Uchar.t) y =
+    Int_replace_polymorphic_compare.compare__local (i x) (i y)
+  ;;
+
+  let equal__local (x : Stdlib.Uchar.t) y =
+    Int_replace_polymorphic_compare.equal__local (i x) (i y)
+  ;;
+
   let max (x : Stdlib.Uchar.t) y = Bool0.select (x >= y) x y
   let min (x : Stdlib.Uchar.t) y = Bool0.select (x <= y) x y
 end
 
 module Float_replace_polymorphic_compare = struct
-  let ( < ) (x : float) y = Poly.( < ) x y
-  let ( <= ) (x : float) y = Poly.( <= ) x y
-  let ( <> ) (x : float) y = Poly.( <> ) x y
-  let ( = ) (x : float) y = Poly.( = ) x y
-  let ( > ) (x : float) y = Poly.( > ) x y
-  let ( >= ) (x : float) y = Poly.( >= ) x y
+  external ( < ) : (float[@local_opt]) -> (float[@local_opt]) -> bool = "%lessthan"
+  external ( <= ) : (float[@local_opt]) -> (float[@local_opt]) -> bool = "%lessequal"
+  external ( <> ) : (float[@local_opt]) -> (float[@local_opt]) -> bool = "%notequal"
+  external ( = ) : (float[@local_opt]) -> (float[@local_opt]) -> bool = "%equal"
+  external ( > ) : (float[@local_opt]) -> (float[@local_opt]) -> bool = "%greaterthan"
+  external ( >= ) : (float[@local_opt]) -> (float[@local_opt]) -> bool = "%greaterequal"
+  external equal : (float[@local_opt]) -> (float[@local_opt]) -> bool = "%equal"
+  external compare : (float[@local_opt]) -> (float[@local_opt]) -> int = "%compare"
+
   let ascending (x : float) y = Poly.ascending x y
   let descending (x : float) y = Poly.descending x y
-  let compare (x : float) y = Poly.compare x y
-  let equal (x : float) y = Poly.equal x y
+  let compare__local (x : float) y = Poly.compare x y
+  let equal__local (x : float) y = Poly.equal x y
   let max (x : float) y = Bool0.select (x >= y) x y
   let min (x : float) y = Bool0.select (x <= y) x y
 end
@@ -262,7 +299,9 @@ module String_replace_polymorphic_compare = struct
   let ascending (x : string) y = Poly.ascending x y
   let descending (x : string) y = Poly.descending x y
   let compare (x : string) y = Poly.compare x y
+  let compare__local (x : string) y = Poly.compare x y
   let equal (x : string) y = Poly.equal x y
+  let equal__local (x : string) y = Poly.equal x y
   let max (x : string) y = Bool0.select (x >= y) x y
   let min (x : string) y = Bool0.select (x <= y) x y
 end
@@ -277,17 +316,19 @@ module Bytes_replace_polymorphic_compare = struct
   let ascending (x : bytes) y = Poly.ascending x y
   let descending (x : bytes) y = Poly.descending x y
   let compare (x : bytes) y = Poly.compare x y
+  let compare__local (x : bytes) y = Poly.compare x y
   let equal (x : bytes) y = Poly.equal x y
+  let equal__local (x : bytes) y = Poly.equal x y
   let max (x : bytes) y = Bool0.select (x >= y) x y
   let min (x : bytes) y = Bool0.select (x <= y) x y
 end
 
 (* This needs to be defined as an external so that the compiler can specialize it as a
-   direct set or caml_modify *)
+   direct set or caml_modify. *)
 external ( := ) : ('a ref[@local_opt]) -> 'a -> unit = "%setfield0"
 
 (* These need to be defined as an external otherwise the compiler won't unbox
-   references *)
+   references. *)
 external ( ! ) : ('a ref[@local_opt]) -> 'a = "%field0"
 external ref : 'a -> ('a ref[@local_opt]) = "%makemutable"
 
@@ -312,13 +353,11 @@ let snd = Stdlib.snd
 (* [raise] needs to be defined as an external as the compiler automatically replaces
    '%raise' by '%reraise' when appropriate. *)
 external raise : exn -> _ = "%raise"
-
-let phys_equal = Stdlib.( == )
-
+external phys_equal : ('a[@local_opt]) -> ('a[@local_opt]) -> bool = "%eq"
 external decr : (int ref[@local_opt]) -> unit = "%decr"
 external incr : (int ref[@local_opt]) -> unit = "%incr"
 
-(* used by sexp_conv, which float0 depends on through option *)
+(* Used by sexp_conv, which float0 depends on through option. *)
 let float_of_string = Stdlib.float_of_string
 
 (* [am_testing] is used in a few places to behave differently when in testing mode, such
