@@ -2,7 +2,14 @@
     implement [Int63] on 32-bit platforms; see [Int63_backends.Emulated]. *)
 
 open! Import
-include Int_intf.S
+
+type t [@@deriving_inline globalize]
+
+val globalize : t -> t
+
+[@@@end]
+
+include Int_intf.S with type t := t
 
 val of_int : int -> t
 val to_int : t -> int option
@@ -20,7 +27,7 @@ val bswap16 : t -> t
 val bswap32 : t -> t
 val bswap48 : t -> t
 
-(*_ exported for Core_kernel *)
+(*_ exported for Core *)
 module W : sig
   val wrap_exn : int64 -> t
   val unwrap : t -> int64
