@@ -7,8 +7,9 @@ module type Round = sig
   type t
 
   (** [round] rounds an int to a multiple of a given [to_multiple_of] argument, according
-      to a direction [dir], with default [dir] being [`Nearest].  [round] will raise if
-      [to_multiple_of <= 0].
+      to a direction [dir], with default [dir] being [`Nearest]. [round] will raise if
+      [to_multiple_of <= 0]. If the result overflows (too far positive or too far
+      negative), [round] returns an incorrect result.
 
       {v
        | `Down    | rounds toward Int.neg_infinity                          |
@@ -36,45 +37,77 @@ module type Round = sig
   val round_nearest : t -> to_multiple_of:t -> t
 end
 
+(** String format for integers, [to_string] / [sexp_of_t] direction only. Includes
+    comparisons and hash functions for [[@@deriving]]. *)
+module type To_string_format = sig
+  type t [@@deriving_inline sexp_of, compare ~localize, hash]
+
+  val sexp_of_t : t -> Sexplib0.Sexp.t
+
+  include Ppx_compare_lib.Comparable.S with type t := t
+  include Ppx_compare_lib.Comparable.S_local with type t := t
+  include Ppx_hash_lib.Hashable.S with type t := t
+
+  [@@@end]
+
+  val to_string : t -> string
+  val to_string_hum : ?delimiter:char -> t -> string
+end
+
+(** String format for integers, including both [to_string] / [sexp_of_t] and [of_string] /
+    [t_of_sexp] directions. Includes comparisons and hash functions for [[@@deriving]]. *)
+module type String_format = sig
+  type t [@@deriving_inline sexp, sexp_grammar, compare ~localize, hash]
+
+  include Sexplib0.Sexpable.S with type t := t
+
+  val t_sexp_grammar : t Sexplib0.Sexp_grammar.t
+
+  include Ppx_compare_lib.Comparable.S with type t := t
+  include Ppx_compare_lib.Comparable.S_local with type t := t
+  include Ppx_hash_lib.Hashable.S with type t := t
+
+  [@@@end]
+
+  include Stringable.S with type t := t
+
+  val to_string_hum : ?delimiter:char -> t -> string
+end
+
+(** Binary format for integers, unsigned and starting with [0b]. *)
+module type Binaryable = sig
+  type t
+
+  module Binary : To_string_format with type t = t
+end
+
+(** Hex format for integers, signed and starting with [0x]. *)
 module type Hexable = sig
   type t
 
-  module Hex : sig
-    type nonrec t = t [@@deriving_inline sexp, compare, hash]
-    include
-      sig
-        [@@@ocaml.warning "-32"]
-        include Ppx_sexp_conv_lib.Sexpable.S with type  t :=  t
-        val compare : t -> t -> int
-        val hash_fold_t :
-          Ppx_hash_lib.Std.Hash.state -> t -> Ppx_hash_lib.Std.Hash.state
-        val hash : t -> Ppx_hash_lib.Std.Hash.hash_value
-      end[@@ocaml.doc "@inline"]
-    [@@@end]
-
-    include Stringable.S with type t := t
-
-    val to_string_hum : ?delimiter:char -> t -> string
-  end
+  module Hex : String_format with type t = t
 end
 
 module type S_common = sig
-  type t [@@deriving_inline hash, sexp]
-  include
-    sig
-      [@@@ocaml.warning "-32"]
-      val hash_fold_t :
-        Ppx_hash_lib.Std.Hash.state -> t -> Ppx_hash_lib.Std.Hash.state
-      val hash : t -> Ppx_hash_lib.Std.Hash.hash_value
-      include Ppx_sexp_conv_lib.Sexpable.S with type  t :=  t
-    end[@@ocaml.doc "@inline"]
+  type t [@@deriving_inline sexp, sexp_grammar]
+
+  include Sexplib0.Sexpable.S with type t := t
+
+  val t_sexp_grammar : t Sexplib0.Sexp_grammar.t
+
   [@@@end]
 
   include Floatable.S with type t := t
   include Intable.S with type t := t
   include Identifiable.S with type t := t
   include Comparable.With_zero with type t := t
+  include Ppx_compare_lib.Comparable.S_local with type t := t
+  include Ppx_compare_lib.Equal.S_local with type t := t
+  include Invariant.S with type t := t
   include Hexable with type t := t
+  include Binaryable with type t := t
+
+  val of_string_opt : string -> t option
 
   (** [delimiter] is an underscore by default. *)
   val to_string_hum : ?delimiter:char -> t -> string
@@ -285,23 +318,70 @@ module type S = sig
   (** [is_pow2 x] returns true iff [x] is a power of 2.  [is_pow2] raises if [x <= 0]. *)
   val is_pow2 : t -> bool
 
+  (** Returns the number of leading zeros in the binary representation of the input, as an
+      integer between 0 and one less than [num_bits].
+
+      The results are unspecified for [t = 0]. *)
+  val clz : t -> int
+
+  (** Returns the number of trailing zeros in the binary representation of the input, as
+      an integer between 0 and one less than [num_bits].
+
+      The results are unspecified for [t = 0]. *)
+  val ctz : t -> int
+
   (** A sub-module designed to be opened to make working with ints more convenient.  *)
   module O : Operators with type t := t
 end
 
-include (
-struct
-  (** Various functors whose type-correctness ensures desired relationships between
-      interfaces. *)
-
-  module Check_O_contained_in_S (M : S) : module type of M.O = M
-  module Check_O_contained_in_S_unbounded (M : S_unbounded) : module type of M.O = M
-  module Check_S_unbounded_in_S (M : S) : S_unbounded = M
-end :
-sig end)
-
 module type Int_without_module_types = sig
-  include S with type t = int
+  (** OCaml's native integer type.
+
+      The number of bits in an integer is platform dependent, being 31-bits on a 32-bit
+      platform, and 63-bits on a 64-bit platform.  [int] is a signed integer type.  [int]s
+      are also subject to overflow, meaning that [Int.max_value + 1 = Int.min_value].
+
+      [int]s always fit in a machine word. *)
+
+  type t = int [@@deriving_inline globalize]
+
+  val globalize : t -> t
+
+  [@@@end]
+
+  include S with type t := t (** @inline *)
+
+  module O : sig
+    (*_ Declared as externals so that the compiler skips the caml_apply_X wrapping even
+      when compiling without cross library inlining. *)
+    external ( + ) : (t[@local_opt]) -> (t[@local_opt]) -> t = "%addint"
+    external ( - ) : (t[@local_opt]) -> (t[@local_opt]) -> t = "%subint"
+    external ( * ) : (t[@local_opt]) -> (t[@local_opt]) -> t = "%mulint"
+    external ( / ) : (t[@local_opt]) -> (t[@local_opt]) -> t = "%divint"
+    external ( ~- ) : (t[@local_opt]) -> t = "%negint"
+    val ( ** ) : t -> t -> t
+    external ( = ) : (t[@local_opt]) -> (t[@local_opt]) -> bool = "%equal"
+    external ( <> ) : (t[@local_opt]) -> (t[@local_opt]) -> bool = "%notequal"
+    external ( < ) : (t[@local_opt]) -> (t[@local_opt]) -> bool = "%lessthan"
+    external ( > ) : (t[@local_opt]) -> (t[@local_opt]) -> bool = "%greaterthan"
+    external ( <= ) : (t[@local_opt]) -> (t[@local_opt]) -> bool = "%lessequal"
+    external ( >= ) : (t[@local_opt]) -> (t[@local_opt]) -> bool = "%greaterequal"
+    external ( land ) : (t[@local_opt]) -> (t[@local_opt]) -> t = "%andint"
+    external ( lor ) : (t[@local_opt]) -> (t[@local_opt]) -> t = "%orint"
+    external ( lxor ) : (t[@local_opt]) -> (t[@local_opt]) -> t = "%xorint"
+    val lnot : t -> t
+    val abs : t -> t
+    external neg : (t[@local_opt]) -> t = "%negint"
+    val zero : t
+    val ( % ) : t -> t -> t
+    val ( /% ) : t -> t -> t
+    val ( // ) : t -> t -> float
+    external ( lsl ) : (t[@local_opt]) -> (int[@local_opt]) -> t = "%lslint"
+    external ( asr ) : (t[@local_opt]) -> (int[@local_opt]) -> t = "%asrint"
+    external ( lsr ) : (t[@local_opt]) -> (int[@local_opt]) -> t = "%lsrint"
+  end
+
+  include module type of O
 
   (** [max_value_30_bits = 2^30 - 1].  It is useful for writing tests that work on both
       64-bit and 32-bit platforms. *)
@@ -322,10 +402,30 @@ module type Int_without_module_types = sig
       These functions return the least-significant bits of the input. In cases
       where optional conversions return [Some x], truncating conversions return [x]. *)
 
-  val of_int32_trunc : int32 -> t
-  val to_int32_trunc : t -> int32
-  val of_int64_trunc : int64 -> t
-  val of_nativeint_trunc : nativeint -> t
+  (*_ Declared as externals so that the compiler skips the caml_apply_X wrapping even when
+    compiling without cross library inlining. *)
+  external to_int32_trunc : (t[@local_opt]) -> (int32[@local_opt]) = "%int32_of_int"
+  external of_int32_trunc : (int32[@local_opt]) -> t = "%int32_to_int"
+  external of_int64_trunc : (int64[@local_opt]) -> t = "%int64_to_int"
+  external of_nativeint_trunc : (nativeint[@local_opt]) -> t = "%nativeint_to_int"
+
+  (** {2 Byte swap operations}
+
+      Byte swap operations reverse the order of bytes in an integer. For
+      example, {!Int32.bswap32} reorders the bottom 32 bits (or 4 bytes),
+      turning [0x1122_3344] to [0x4433_2211]. Byte swap functions exposed by
+      Base use OCaml primitives to generate assembly instructions to perform
+      the relevant byte swaps.
+
+      For a more extensive list of byteswap functions, see {!Int32} and
+      {!Int64}.
+  *)
+
+  (** Byte swaps bottom 16 bits (2 bytes). The values of the remaining bytes
+      are undefined. *)
+  external bswap16 : (int[@local_opt]) -> int = "%bswap16"
+  (*_ Declared as an external so that the compiler skips the caml_apply_X wrapping even
+    when compiling without cross library inlining. *)
 
   (**/**)
 
@@ -342,19 +442,13 @@ module type Int_without_module_types = sig
   end
 end
 
-(** OCaml's native integer type.
-
-    The number of bits in an integer is platform dependent, being 31-bits on a 32-bit
-    platform, and 63-bits on a 64-bit platform.  [int] is a signed integer type.  [int]s
-    are also subject to overflow, meaning that [Int.max_value + 1 = Int.min_value].
-
-    [int]s always fit in a machine word. *)
 module type Int = sig
-  include Int_without_module_types
+  include Int_without_module_types (** @inline *)
 
   (** {2 Module types specifying integer operations.} *)
-  module type Hexable = Hexable
 
+  module type Binaryable = Binaryable
+  module type Hexable = Hexable
   module type Int_without_module_types = Int_without_module_types
   module type Operators = Operators
   module type Operators_unbounded = Operators_unbounded
@@ -362,4 +456,6 @@ module type Int = sig
   module type S = S
   module type S_common = S_common
   module type S_unbounded = S_unbounded
+  module type String_format = String_format
+  module type To_string_format = To_string_format
 end
