@@ -14,24 +14,31 @@ let t_sexp_grammar : 'a. 'a Sexplib0.Sexp_grammar.t -> 'a t Sexplib0.Sexp_gramma
 
 external force : ('a t[@local_opt]) -> 'a = "%lazy_force"
 
+let globalize = Globalize.globalize_lazy_t
 let map t ~f = lazy (f (force t))
 
-let compare compare_a t1 t2 =
+let compare__local compare_a t1 t2 =
   if phys_equal t1 t2 then 0 else compare_a (force t1) (force t2)
 ;;
 
-let equal equal_a t1 t2 = if phys_equal t1 t2 then true else equal_a (force t1) (force t2)
+let compare compare_a t1 t2 = compare__local compare_a t1 t2
+
+let equal__local equal_a t1 t2 =
+  if phys_equal t1 t2 then true else equal_a (force t1) (force t2)
+;;
+
+let equal equal_a t1 t2 = equal__local equal_a t1 t2
 let hash_fold_t = Hash.Builtin.hash_fold_lazy_t
 let peek t = if is_val t then Some (force t) else None
 
 include Monad.Make (struct
-    type nonrec 'a t = 'a t
+  type nonrec 'a t = 'a t
 
-    let return x = from_val x
-    let bind t ~f = lazy (force (f (force t)))
-    let map = map
-    let map = `Custom map
-  end)
+  let return x = from_val x
+  let bind t ~f = lazy (force (f (force t)))
+  let map = map
+  let map = `Custom map
+end)
 
 module T_unforcing = struct
   type nonrec 'a t = 'a t
