@@ -10,10 +10,16 @@ open! Import
 
 (** Serialization and comparison of an [Error] force the error's lazy message. *)
 type 'a t = ('a, Error.t) Result.t
-[@@deriving_inline compare, equal, hash, sexp, sexp_grammar]
+[@@deriving_inline
+  compare ~localize, equal ~localize, globalize, hash, sexp, sexp_grammar]
 
 include Ppx_compare_lib.Comparable.S1 with type 'a t := 'a t
+include Ppx_compare_lib.Comparable.S_local1 with type 'a t := 'a t
 include Ppx_compare_lib.Equal.S1 with type 'a t := 'a t
+include Ppx_compare_lib.Equal.S_local1 with type 'a t := 'a t
+
+val globalize : ('a -> 'a) -> 'a t -> 'a t
+
 include Ppx_hash_lib.Hashable.S1 with type 'a t := 'a t
 include Sexplib0.Sexpable.S1 with type 'a t := 'a t
 
@@ -37,15 +43,9 @@ val is_error : _ t -> bool
     [Error.t].  [try_with_join] is like [try_with], except that [f] can throw exceptions
     or return an [Error] directly, without ending up with a nested error; it is equivalent
     to [Result.join (try_with f)]. *)
-val try_with
-  :  ?backtrace:bool (** defaults to [false] *)
-  -> ((unit -> 'a)[@local])
-  -> 'a t
+val try_with : ?backtrace:bool (** defaults to [false] *) -> (unit -> 'a) -> 'a t
 
-val try_with_join
-  :  ?backtrace:bool (** defaults to [false] *)
-  -> ((unit -> 'a t)[@local])
-  -> 'a t
+val try_with_join : ?backtrace:bool (** defaults to [false] *) -> (unit -> 'a t) -> 'a t
 
 (** [ok t] returns [None] if [t] is an [Error], and otherwise returns the contents of the
     [Ok] constructor. *)
@@ -63,6 +63,10 @@ val of_exn : ?backtrace:[ `Get | `This of string ] -> exn -> _ t
     [of_exn_result ?backtrace (Error exn) = of_exn ?backtrace exn] *)
 val of_exn_result : ?backtrace:[ `Get | `This of string ] -> ('a, exn) Result.t -> 'a t
 
+(** [of_option t] returns [Ok 'a] if [t] is [Some 'a], and otherwise returns the supplied
+    [error] as [Error error] *)
+val of_option : 'a option -> error:Error.t -> 'a t
+
 (** [error] is a wrapper around [Error.create]:
 
     {[
@@ -110,9 +114,9 @@ val tag_arg : 'a t -> string -> 'b -> ('b -> Sexp.t) -> 'a t
     the function that is unimplemented. *)
 val unimplemented : string -> _ t
 
-val map : 'a t -> f:(('a -> 'b)[@local]) -> 'b t
-val iter : 'a t -> f:(('a -> unit)[@local]) -> unit
-val iter_error : _ t -> f:((Error.t -> unit)[@local]) -> unit
+val map : 'a t -> f:('a -> 'b) -> 'b t
+val iter : 'a t -> f:('a -> unit) -> unit
+val iter_error : _ t -> f:(Error.t -> unit) -> unit
 
 (** [combine_errors ts] returns [Ok] if every element in [ts] is [Ok], else it returns
     [Error] with all the errors in [ts].  More precisely:
@@ -136,4 +140,4 @@ val find_ok : 'a t list -> 'a t
 
 (** [find_map_ok l ~f] returns the first value in [l] for which [f] returns [Ok],
     otherwise it returns the same error as [combine_errors (List.map l ~f)]. *)
-val find_map_ok : 'a list -> f:(('a -> 'b t)[@local]) -> 'b t
+val find_map_ok : 'a list -> f:('a -> 'b t) -> 'b t
