@@ -2,12 +2,12 @@ open! Import
 include Sexplib0.Sexpable
 
 module Of_sexpable
-    (Sexpable : S) (M : sig
-                      type t
+  (Sexpable : S) (M : sig
+    type t
 
-                      val to_sexpable : t -> Sexpable.t
-                      val of_sexpable : Sexpable.t -> t
-                    end) : S with type t := M.t = struct
+    val to_sexpable : t -> Sexpable.t
+    val of_sexpable : Sexpable.t -> t
+  end) : S with type t := M.t = struct
   let t_of_sexp sexp =
     let s = Sexpable.t_of_sexp sexp in
     try M.of_sexpable s with
@@ -18,12 +18,12 @@ module Of_sexpable
 end
 
 module Of_sexpable1
-    (Sexpable : S1) (M : sig
-                       type 'a t
+  (Sexpable : S1) (M : sig
+    type 'a t
 
-                       val to_sexpable : 'a t -> 'a Sexpable.t
-                       val of_sexpable : 'a Sexpable.t -> 'a t
-                     end) : S1 with type 'a t := 'a M.t = struct
+    val to_sexpable : 'a t -> 'a Sexpable.t
+    val of_sexpable : 'a Sexpable.t -> 'a t
+  end) : S1 with type 'a t := 'a M.t = struct
   let t_of_sexp a_of_sexp sexp =
     let s = Sexpable.t_of_sexp a_of_sexp sexp in
     try M.of_sexpable s with
@@ -34,12 +34,12 @@ module Of_sexpable1
 end
 
 module Of_sexpable2
-    (Sexpable : S2) (M : sig
-                       type ('a, 'b) t
+  (Sexpable : S2) (M : sig
+    type ('a, 'b) t
 
-                       val to_sexpable : ('a, 'b) t -> ('a, 'b) Sexpable.t
-                       val of_sexpable : ('a, 'b) Sexpable.t -> ('a, 'b) t
-                     end) : S2 with type ('a, 'b) t := ('a, 'b) M.t = struct
+    val to_sexpable : ('a, 'b) t -> ('a, 'b) Sexpable.t
+    val of_sexpable : ('a, 'b) Sexpable.t -> ('a, 'b) t
+  end) : S2 with type ('a, 'b) t := ('a, 'b) M.t = struct
   let t_of_sexp a_of_sexp b_of_sexp sexp =
     let s = Sexpable.t_of_sexp a_of_sexp b_of_sexp sexp in
     try M.of_sexpable s with
@@ -52,12 +52,12 @@ module Of_sexpable2
 end
 
 module Of_sexpable3
-    (Sexpable : S3) (M : sig
-                       type ('a, 'b, 'c) t
+  (Sexpable : S3) (M : sig
+    type ('a, 'b, 'c) t
 
-                       val to_sexpable : ('a, 'b, 'c) t -> ('a, 'b, 'c) Sexpable.t
-                       val of_sexpable : ('a, 'b, 'c) Sexpable.t -> ('a, 'b, 'c) t
-                     end) : S3 with type ('a, 'b, 'c) t := ('a, 'b, 'c) M.t = struct
+    val to_sexpable : ('a, 'b, 'c) t -> ('a, 'b, 'c) Sexpable.t
+    val of_sexpable : ('a, 'b, 'c) Sexpable.t -> ('a, 'b, 'c) t
+  end) : S3 with type ('a, 'b, 'c) t := ('a, 'b, 'c) M.t = struct
   let t_of_sexp a_of_sexp b_of_sexp c_of_sexp sexp =
     let s = Sexpable.t_of_sexp a_of_sexp b_of_sexp c_of_sexp sexp in
     try M.of_sexpable s with
@@ -69,7 +69,16 @@ module Of_sexpable3
   ;;
 end
 
-module Of_stringable (M : Stringable.S) : S with type t := M.t = struct
+module Of_stringable (M : Stringable.S) : sig
+  type t [@@deriving_inline sexp_grammar]
+
+  val t_sexp_grammar : t Sexplib0.Sexp_grammar.t
+
+  [@@@end]
+
+  include S with type t := t
+end
+with type t := M.t = struct
   let t_of_sexp sexp =
     match sexp with
     | Sexp.Atom s ->
@@ -82,4 +91,8 @@ module Of_stringable (M : Stringable.S) : S with type t := M.t = struct
   ;;
 
   let sexp_of_t t = Sexp.Atom (M.to_string t)
+
+  let t_sexp_grammar : M.t Sexplib0.Sexp_grammar.t =
+    Sexplib0.Sexp_grammar.coerce string_sexp_grammar
+  ;;
 end
