@@ -10,6 +10,5 @@ let%expect_test _ =
   Backtrace.elide := true;
   Stdio.Out_channel.(output_string stdout)
     (Sexp.to_string (sexp_of_t (Exn.with_recording false ~f:Exn.most_recent)));
-  [%expect {|
-    ("<backtrace elided in test>") |}]
+  [%expect {| ("<backtrace elided in test>") |}]
 ;;
