@@ -518,7 +518,8 @@ let%test_module "Assoc.group" =
       test [ "a", 1; "b", 2 ];
       [%expect {|
         ((a (1))
-         (b (2))) |}];
+         (b (2)))
+        |}];
       test [ "odd", 1; "even", 2; "Odd", 3; "Even", 4; "ODD", 5; "EVEN", 6 ];
       [%expect
         {|
@@ -527,11 +528,13 @@ let%test_module "Assoc.group" =
          (Odd  (3))
          (Even (4))
          (ODD  (5))
-         (EVEN (6))) |}];
+         (EVEN (6)))
+        |}];
       test [ "odd", 1; "Odd", 3; "ODD", 5; "even", 2; "Even", 4; "EVEN", 6 ];
       [%expect {|
         ((odd  (1 3 5))
-         (even (2 4 6))) |}]
+         (even (2 4 6)))
+        |}]
     ;;
   end)
 ;;
@@ -557,11 +560,13 @@ let%test_module "Assoc.sort_and_group" =
       test [ "a", 1; "b", 2 ];
       [%expect {|
         ((a (1))
-         (b (2))) |}];
+         (b (2)))
+        |}];
       test [ "odd", 1; "even", 2; "Odd", 3; "Even", 4; "ODD", 5; "EVEN", 6 ];
       [%expect {|
         ((even (2 4 6))
-         (odd  (1 3 5))) |}]
+         (odd  (1 3 5)))
+        |}]
     ;;
   end)
 ;;
@@ -954,29 +959,15 @@ let%test_unit _ =
 ;;
 
 let%test_unit _ =
-  [%test_result: int]
-    (length
-       (find_all_dups
-          [ 0, 1; 2, 2; 0, 2; 4, 1 ]
-          ~compare:(fun (_, a) (_, b) -> Int.compare a b)))
-    ~expect:2
+  [%test_result: (int * int) list]
+    (find_all_dups [ 0, 1; 2, 2; 0, 2; 4, 1 ] ~compare:[%compare: _ * int])
+    ~expect:[ 4, 1; 0, 2 ]
 ;;
 
 let%test_unit _ =
-  [%test_result: int]
-    (length
-       (find_all_dups
-          [ 0, 1; 2, 2; 0, 2; 4, 1 ]
-          ~compare:(fun (a, _) (b, _) -> Int.compare a b)))
-    ~expect:1
-;;
-
-let%test_unit _ =
-  [%test_result: int] (counti [ 0; 1; 2; 3; 4 ] ~f:(fun idx x -> idx = x)) ~expect:5
-;;
-
-let%test_unit _ =
-  [%test_result: int] (counti [ 0; 1; 2; 3; 4 ] ~f:(fun idx x -> idx = 4 - x)) ~expect:1
+  [%test_result: (int * int) list]
+    (find_all_dups [ 0, 1; 2, 2; 0, 2; 4, 1 ] ~compare:[%compare: int * _])
+    ~expect:[ 0, 2 ]
 ;;
 
 let%test_unit _ =
@@ -1375,5 +1366,581 @@ let%expect_test "[Cartesian_product]" =
     (c 2 mi)
     (c 3 do)
     (c 3 re)
-    (c 3 mi) |}]
+    (c 3 mi)
+    |}]
+;;
+
+let%expect_test "[compare__local] is the same as [compare]" =
+  Base_quickcheck.Test.run_exn
+    (module struct
+      type t = int list * int list [@@deriving sexp_of, quickcheck]
+    end)
+    ~f:(fun (l1, l2) ->
+      require_equal
+        [%here]
+        (module Int)
+        (compare Int.compare l1 l2)
+        (compare__local Int.compare__local l1 l2));
+  [%expect {| |}]
+;;
+
+let%expect_test "[equal__local] is the same as [equal]" =
+  Base_quickcheck.Test.run_exn
+    (module struct
+      type t = int list * int list [@@deriving sexp_of, quickcheck]
+    end)
+    ~f:(fun (l1, l2) ->
+      require_equal
+        [%here]
+        (module Bool)
+        (equal Int.equal l1 l2)
+        (equal__local Int.equal__local l1 l2));
+  [%expect {| |}]
+;;
+
+let%expect_test "list sort, dedup" =
+  let slow_stable_sort list ~compare =
+    let rec insert elt list =
+      match list with
+      | [] -> [ elt ]
+      | head :: tail ->
+        (match compare elt head with
+         | c when c <= 0 -> elt :: list
+         | _ -> head :: insert elt tail)
+    in
+    List.fold_right list ~init:[] ~f:insert
+  in
+  let slow_sort list ~compare =
+    (* sort happens to behave the same as stable_sort *)
+    let rec insert elt list =
+      match list with
+      | [] -> [ elt ]
+      | head :: tail ->
+        (match compare elt head with
+         | c when c <= 0 -> elt :: list
+         | _ -> head :: insert elt tail)
+    in
+    List.fold_right list ~init:[] ~f:insert
+  in
+  let slow_dedup_and_sort list ~compare =
+    (* dedup_and_sort keeps the last element among duplicates *)
+    let rec insert elt list =
+      match list with
+      | [] -> [ elt ]
+      | head :: tail ->
+        (match compare elt head with
+         | 0 -> list
+         | c when c < 0 -> elt :: list
+         | _ -> head :: insert elt tail)
+    in
+    List.fold_right list ~init:[] ~f:insert
+  in
+  let slow_stable_dedup list ~compare =
+    (* stable_dedup keeps the first element among duplicates *)
+    let insert elt list =
+      elt :: List.filter list ~f:(fun other -> compare elt other <> 0)
+    in
+    List.fold_right list ~init:[] ~f:insert
+  in
+  let module Char_list = struct
+    open Base_quickcheck
+
+    type t = char list [@@deriving equal, sexp_of]
+
+    let quickcheck_generator = Generator.list_non_empty Generator.char_alpha
+
+    let quickcheck_shrinker =
+      let prev char = Char.of_int_exn (Int.pred (Char.to_int char)) in
+      Shrinker.list
+        (Shrinker.create (function
+          | 'a' -> Sequence.empty
+          | 'b' .. 'z' as char -> Sequence.singleton (prev char)
+          | 'A' -> Sequence.singleton 'a'
+          | 'B' .. 'Z' as char -> Sequence.of_list [ Char.lowercase char; prev char ]
+          | _ -> Sequence.empty))
+    ;;
+  end
+  in
+  let compare = Char.Caseless.compare in
+  quickcheck_m
+    [%here]
+    ~examples:[ []; [ 'a'; 'a' ]; [ 'a'; 'A' ]; [ 'A'; 'a' ]; [ 'A'; 'A' ] ]
+    (module Char_list)
+    ~f:(fun list ->
+      require_equal
+        [%here]
+        (module Char_list)
+        ~message:"sort mismatch"
+        (List.sort list ~compare)
+        (slow_sort list ~compare);
+      require_equal
+        [%here]
+        (module Char_list)
+        ~message:"stable_sort mismatch"
+        (List.stable_sort list ~compare)
+        (slow_stable_sort list ~compare);
+      require_equal
+        [%here]
+        (module Char_list)
+        ~message:"dedup_and_sort mismatch"
+        (List.dedup_and_sort list ~compare)
+        (slow_dedup_and_sort list ~compare);
+      require_equal
+        [%here]
+        (module Char_list)
+        ~message:"stable_dedup mismatch"
+        (List.stable_dedup list ~compare)
+        (slow_stable_dedup list ~compare))
+;;
+
+let%expect_test "[take], [drop], and [split]" =
+  for whole_len = 0 to 3 do
+    let whole = List.init whole_len ~f:Fn.id in
+    let test name kind requested_length list =
+      let expected_length = Int.clamp_exn requested_length ~min:0 ~max:whole_len in
+      let phys_equal_whole = phys_equal list whole in
+      let check problem bool =
+        require
+          [%here]
+          bool
+          ~if_false_then_print_s:
+            [%lazy_message
+              name
+                ~problem
+                (whole : int list)
+                (requested_length : int)
+                (expected_length : int)
+                (phys_equal_whole : bool)
+                (list : int list)]
+      in
+      check "wrong length" (List.length list = expected_length);
+      if expected_length = whole_len then check "not phys_equal" phys_equal_whole;
+      match kind with
+      | `prefix ->
+        check "is not a prefix" (List.is_prefix whole ~prefix:list ~equal:( = ))
+      | `suffix ->
+        check "is not a suffix" (List.is_suffix whole ~suffix:list ~equal:( = ))
+    in
+    for prefix_len = -1 to 2 do
+      let suffix_len = whole_len - prefix_len in
+      test "take" `prefix prefix_len (List.take whole prefix_len);
+      test "drop" `suffix suffix_len (List.drop whole prefix_len);
+      let prefix, suffix = List.split_n whole prefix_len in
+      test "split prefix" `prefix prefix_len prefix;
+      test "split suffix" `suffix suffix_len suffix
+    done
+  done;
+  [%expect {| |}]
+;;
+
+let print_s sexp =
+  Ref.set_temporarily sexp_style Sexp_style.simple_pretty ~f:(fun () -> print_s sexp)
+;;
+
+let%expect_test "[cartesian_product]" =
+  let test xs ys = print_s [%sexp (cartesian_product xs ys : (int * int) list)] in
+  test [] [];
+  [%expect {| () |}];
+  test [ 1; 2; 3 ] [];
+  [%expect {| () |}];
+  test [] [ 1; 2; 3 ];
+  [%expect {| () |}];
+  test [ 1 ] [ 2; 3; 4 ];
+  [%expect {| ((1 2) (1 3) (1 4)) |}];
+  test [ 1; 2; 3 ] [ 4 ];
+  [%expect {| ((1 4) (2 4) (3 4)) |}];
+  test [ 1; 2 ] [ 3; 4; 5 ];
+  [%expect {| ((1 3) (1 4) (1 5) (2 3) (2 4) (2 5)) |}];
+  test [ 1; 2; 3 ] [ 4; 5 ];
+  [%expect {| ((1 4) (1 5) (2 4) (2 5) (3 4) (3 5)) |}];
+  test [ 1; 2; 3 ] [ 4; 5; 6 ];
+  [%expect {| ((1 4) (1 5) (1 6) (2 4) (2 5) (2 6) (3 4) (3 5) (3 6)) |}]
+;;
+
+let%expect_test "[concat_map]" =
+  let test list =
+    List.concat_map list ~f:(fun n -> List.init n ~f:Int.succ)
+    |> [%sexp_of: int list]
+    |> print_s
+  in
+  test [];
+  [%expect {| () |}];
+  test [ 1 ];
+  [%expect {| (1) |}];
+  test [ 1; 2 ];
+  [%expect {| (1 1 2) |}];
+  test [ 1; 2; 3 ];
+  [%expect {| (1 1 2 1 2 3) |}];
+  test [ 1; 2; 3; 4 ];
+  [%expect {| (1 1 2 1 2 3 1 2 3 4) |}];
+  test [ 4; 5; 6 ];
+  [%expect {| (1 2 3 4 1 2 3 4 5 1 2 3 4 5 6) |}]
+;;
+
+let%expect_test "[concat_mapi]" =
+  let test list =
+    List.concat_mapi list ~f:(fun i n -> List.init n ~f:(( + ) i))
+    |> [%sexp_of: int list]
+    |> print_s
+  in
+  test [];
+  [%expect {| () |}];
+  test [ 1 ];
+  [%expect {| (0) |}];
+  test [ 1; 2 ];
+  [%expect {| (0 1 2) |}];
+  test [ 1; 2; 3 ];
+  [%expect {| (0 1 2 2 3 4) |}];
+  test [ 1; 2; 3; 4 ];
+  [%expect {| (0 1 2 2 3 4 3 4 5 6) |}];
+  test [ 4; 5; 6 ];
+  [%expect {| (0 1 2 3 1 2 3 4 5 2 3 4 5 6 7) |}]
+;;
+
+let%test_module "filter{,i}" =
+  (module struct
+    open Base_quickcheck
+
+    module Int_list = struct
+      type t = int list [@@deriving equal, sexp_of]
+    end
+
+    let%expect_test "[filter]" =
+      quickcheck_m
+        [%here]
+        (module struct
+          type t = int list * (int -> bool) [@@deriving quickcheck, sexp_of]
+        end)
+        ~f:(fun (list, f) ->
+          (* test [f] *)
+          let pos = List.filter list ~f in
+          require [%here] (List.for_all pos ~f);
+          (* test [~f] *)
+          let not_f = Fn.non f in
+          let neg = List.filter list ~f:not_f in
+          require [%here] (List.for_all neg ~f:not_f);
+          (* test [f \/ ~f] *)
+          let sort = sort ~compare:Int.compare in
+          require_equal [%here] (module Int_list) (sort list) (sort (pos @ neg)))
+    ;;
+
+    let%expect_test "[filteri]" =
+      quickcheck_m
+        [%here]
+        (module struct
+          type t = int list * (int -> int -> bool) [@@deriving quickcheck, sexp_of]
+        end)
+        ~f:(fun (list, f) ->
+          let pos, neg =
+            (* stash the original indices, so that we can retrieve them after filtering *)
+            let list = mapi list ~f:(fun i x -> i, x) in
+            let ignore_stash f : _ = fun i (_, x) -> f i x in
+            let use_orig_index f : _ = fun (i, x) -> f i x in
+            (* test [f] *)
+            let pos = List.filteri list ~f:(ignore_stash f) in
+            require [%here] (List.for_all pos ~f:(use_orig_index f));
+            (* test [~f] *)
+            let not_f i x = not (f i x) in
+            let neg = List.filteri list ~f:(ignore_stash not_f) in
+            require [%here] (List.for_all neg ~f:(use_orig_index not_f));
+            pos, neg
+          in
+          (* test [f \/ ~f] *)
+          let sort = sort ~compare:[%compare: int * _] in
+          require_equal [%here] (module Int_list) list (sort (pos @ neg) |> map ~f:snd))
+    ;;
+
+    let%expect_test "[filteri ~f:(Fn.const f) = filter ~f]" =
+      quickcheck_m
+        [%here]
+        (module struct
+          type t = int list * (int -> bool) [@@deriving quickcheck, sexp_of]
+        end)
+        ~f:(fun (list, f) ->
+          require_equal
+            [%here]
+            (module Int_list)
+            (filteri list ~f:(fun _ x -> f x))
+            (filter list ~f))
+    ;;
+
+    let%expect_test "[filter]" =
+      let test list =
+        List.filter list ~f:(fun n -> n % 3 > 0) |> [%sexp_of: int list] |> print_s
+      in
+      test [];
+      [%expect {| () |}];
+      test [ 1 ];
+      [%expect {| (1) |}];
+      test [ 1; 2 ];
+      [%expect {| (1 2) |}];
+      test [ 1; 2; 3 ];
+      [%expect {| (1 2) |}];
+      test [ 1; 2; 3; 4 ];
+      [%expect {| (1 2 4) |}];
+      test [ 4; 5; 6 ];
+      [%expect {| (4 5) |}]
+    ;;
+
+    let%expect_test "[filteri]" =
+      let test list =
+        List.filteri list ~f:(fun i n -> n > i) |> [%sexp_of: int list] |> print_s
+      in
+      test [];
+      [%expect {| () |}];
+      test [ 0 ];
+      [%expect {| () |}];
+      test [ 0; 1 ];
+      [%expect {| () |}];
+      test [ 0; 1; 2 ];
+      [%expect {| () |}];
+      test [ 1 ];
+      [%expect {| (1) |}];
+      test [ 1; 2 ];
+      [%expect {| (1 2) |}];
+      test [ 1; 2; 3 ];
+      [%expect {| (1 2 3) |}];
+      test [ 1; 0 ];
+      [%expect {| (1) |}];
+      test [ 2; 1; 0 ];
+      [%expect {| (2) |}];
+      test [ 3; 2; 1; 0 ];
+      [%expect {| (3 2) |}]
+    ;;
+  end)
+;;
+
+let%test_module "count{,i}" =
+  (module struct
+    let%expect_test "[count{,i} list ~f = List.length (filter{,i} list ~f)]" =
+      quickcheck_m
+        [%here]
+        (module struct
+          type t = int list * (int -> bool) [@@deriving quickcheck, sexp_of]
+        end)
+        ~f:(fun (list, f) ->
+          require_equal [%here] (module Int) (count list ~f) (length (filter list ~f)));
+      quickcheck_m
+        [%here]
+        (module struct
+          type t = int list * (int -> int -> bool) [@@deriving quickcheck, sexp_of]
+        end)
+        ~f:(fun (list, f) ->
+          require_equal [%here] (module Int) (counti list ~f) (length (filteri list ~f)))
+    ;;
+
+    let%test_unit _ =
+      [%test_result: int] (counti [ 0; 1; 2; 3; 4 ] ~f:(fun idx x -> idx = x)) ~expect:5
+    ;;
+
+    let%test_unit _ =
+      [%test_result: int]
+        (counti [ 0; 1; 2; 3; 4 ] ~f:(fun idx x -> idx = 4 - x))
+        ~expect:1
+    ;;
+  end)
+;;
+
+let%test_module "{min,max}_elt" =
+  (module struct
+    let test_in_list_and_forall ~tested_f ~holds_for_res_over_all_elem =
+      quickcheck_m
+        [%here]
+        (module struct
+          type t = int list [@@deriving quickcheck, sexp_of]
+        end)
+        ~f:(fun list ->
+          let res = tested_f list ~compare:[%compare: int] in
+          match res with
+          | None -> require [%here] (is_empty list)
+          | Some res ->
+            require [%here] (mem list res ~equal:Int.equal);
+            iter list ~f:(fun elem ->
+              require [%here] (holds_for_res_over_all_elem ~res ~elem)))
+    ;;
+
+    let%expect_test "min_elt" =
+      test_in_list_and_forall
+        ~tested_f:min_elt
+        ~holds_for_res_over_all_elem:(fun ~res ~elem -> res <= elem)
+    ;;
+
+    let%expect_test "max_elt" =
+      test_in_list_and_forall
+        ~tested_f:max_elt
+        ~holds_for_res_over_all_elem:(fun ~res ~elem -> res >= elem)
+    ;;
+  end)
+;;
+
+let%expect_test "[map2]" =
+  let test xs ys =
+    map2 xs ys ~f:(fun x y -> x, y)
+    |> [%sexp_of: (int * int) list Or_unequal_lengths.t]
+    |> print_s
+  in
+  test [] [];
+  [%expect {| (Ok ()) |}];
+  test [ 1 ] [ 2 ];
+  [%expect {| (Ok ((1 2))) |}];
+  test [ 1; 2 ] [ 3; 4 ];
+  [%expect {| (Ok ((1 3) (2 4))) |}];
+  test [ 1; 2; 3 ] [ 4; 5; 6 ];
+  [%expect {| (Ok ((1 4) (2 5) (3 6))) |}];
+  test [] [ 1 ];
+  [%expect {| Unequal_lengths |}];
+  test [ 1 ] [];
+  [%expect {| Unequal_lengths |}];
+  test [ 1 ] [ 2; 3; 4 ];
+  [%expect {| Unequal_lengths |}];
+  test [ 1; 2; 3 ] [ 4 ];
+  [%expect {| Unequal_lengths |}]
+;;
+
+let%expect_test "[map3]" =
+  let test xs ys zs =
+    map3 xs ys zs ~f:(fun x y z -> x, y, z)
+    |> [%sexp_of: (int * int * int) list Or_unequal_lengths.t]
+    |> print_s
+  in
+  test [] [] [];
+  [%expect {| (Ok ()) |}];
+  test [ 1 ] [ 2 ] [ 3 ];
+  [%expect {| (Ok ((1 2 3))) |}];
+  test [ 1; 2 ] [ 3; 4 ] [ 5; 6 ];
+  [%expect {| (Ok ((1 3 5) (2 4 6))) |}];
+  test [ 1; 2; 3 ] [ 4; 5; 6 ] [ 7; 8; 9 ];
+  [%expect {| (Ok ((1 4 7) (2 5 8) (3 6 9))) |}];
+  test [] [] [ 1 ];
+  [%expect {| Unequal_lengths |}];
+  test [] [ 1 ] [];
+  [%expect {| Unequal_lengths |}];
+  test [] [] [ 1 ];
+  [%expect {| Unequal_lengths |}];
+  test [ 1 ] [ 2 ] [ 3; 4; 5 ];
+  [%expect {| Unequal_lengths |}];
+  test [ 1 ] [ 2; 3; 4 ] [ 5 ];
+  [%expect {| Unequal_lengths |}];
+  test [ 1; 2; 3 ] [ 4 ] [ 5 ];
+  [%expect {| Unequal_lengths |}]
+;;
+
+let%expect_test "[merge]" =
+  let module Int_list = struct
+    type t = int list [@@deriving equal, sexp_of]
+  end
+  in
+  let test_int xs ys =
+    let list1 = merge xs ys ~compare:Int.compare in
+    print_s [%sexp (list1 : int list)];
+    let list2 = merge ys xs ~compare:Int.compare in
+    require_equal [%here] (module Int_list) list1 list2
+  in
+  test_int [] [];
+  [%expect {| () |}];
+  test_int [] [ 1; 2; 3 ];
+  [%expect {| (1 2 3) |}];
+  test_int [ 1; 2 ] [ 3; 4; 5 ];
+  [%expect {| (1 2 3 4 5) |}];
+  test_int [ 1; 3 ] [ 2; 4; 5 ];
+  [%expect {| (1 2 3 4 5) |}];
+  test_int [ 1; 4 ] [ 2; 3; 5 ];
+  [%expect {| (1 2 3 4 5) |}];
+  test_int [ 1; 5 ] [ 2; 3; 4 ];
+  [%expect {| (1 2 3 4 5) |}];
+  test_int [ 1; 3; 5 ] [ 2; 4 ];
+  [%expect {| (1 2 3 4 5) |}];
+  test_int [ 1; 3; 4 ] [ 1; 2 ];
+  [%expect {| (1 1 2 3 4) |}];
+  let test_pair xs ys =
+    let list1 = merge xs ys ~compare:[%compare: int * _] in
+    print_s [%sexp (list1 : (int * string) list)];
+    let list2 = merge ys xs ~compare:[%compare: int * _] in
+    print_s [%sexp (list2 : (int * string) list)]
+  in
+  test_pair [] [];
+  [%expect {|
+    ()
+    ()
+    |}];
+  test_pair [] [ 1, "a"; 2, "b"; 3, "c" ];
+  [%expect {|
+    ((1 a) (2 b) (3 c))
+    ((1 a) (2 b) (3 c))
+    |}];
+  test_pair [ 1, "z"; 2, "y" ] [ 3, "x"; 4, "w"; 5, "v" ];
+  [%expect
+    {|
+    ((1 z) (2 y) (3 x) (4 w) (5 v))
+    ((1 z) (2 y) (3 x) (4 w) (5 v))
+    |}];
+  test_pair [ 1, "a"; 2, "b" ] [];
+  [%expect {|
+    ((1 a) (2 b))
+    ((1 a) (2 b))
+    |}];
+  test_pair [ 1, "a"; 3, "b" ] [ 1, "b"; 2, "a" ];
+  [%expect {|
+    ((1 a) (1 b) (2 a) (3 b))
+    ((1 b) (1 a) (2 a) (3 b))
+    |}];
+  test_pair [ 0, "!"; 1, "b"; 2, "a" ] [ 1, "a"; 2, "b" ];
+  [%expect
+    {|
+    ((0 !) (1 b) (1 a) (2 a) (2 b))
+    ((0 !) (1 a) (1 b) (2 b) (2 a))
+    |}]
+;;
+
+let%expect_test "[sub]" =
+  let test pos len list =
+    match sub list ~pos ~len with
+    | list -> print_s [%sexp (list : int list)]
+    | exception exn -> print_s [%sexp "raised", (exn : exn)]
+  in
+  test 0 0 [];
+  [%expect {| () |}];
+  test 1 0 [];
+  [%expect {| (raised (Invalid_argument List.sub)) |}];
+  test 0 1 [];
+  [%expect {| (raised (Invalid_argument List.sub)) |}];
+  let list = [ 1; 2; 3; 4 ] in
+  test 0 0 list;
+  [%expect {| () |}];
+  test 0 4 list;
+  [%expect {| (1 2 3 4) |}];
+  test 0 1 list;
+  [%expect {| (1) |}];
+  test 1 1 list;
+  [%expect {| (2) |}];
+  test 2 1 list;
+  [%expect {| (3) |}];
+  test 3 1 list;
+  [%expect {| (4) |}];
+  test 4 1 list;
+  [%expect {| (raised (Invalid_argument List.sub)) |}];
+  test 0 2 list;
+  [%expect {| (1 2) |}];
+  test 1 2 list;
+  [%expect {| (2 3) |}];
+  test 2 2 list;
+  [%expect {| (3 4) |}];
+  test 3 2 list;
+  [%expect {| (raised (Invalid_argument List.sub)) |}];
+  test 0 3 list;
+  [%expect {| (1 2 3) |}];
+  test 1 3 list;
+  [%expect {| (2 3 4) |}];
+  test 2 3 list;
+  [%expect {| (raised (Invalid_argument List.sub)) |}];
+  test 0 4 list;
+  [%expect {| (1 2 3 4) |}];
+  test 1 4 list;
+  [%expect {| (raised (Invalid_argument List.sub)) |}];
+  test 0 5 list;
+  [%expect {| (raised (Invalid_argument List.sub)) |}];
+  test (-1) 0 list;
+  [%expect {| (raised (Invalid_argument List.sub)) |}];
+  test 1 (-1) list;
+  [%expect {| (raised (Invalid_argument List.sub)) |}]
 ;;
