@@ -58,7 +58,7 @@ let%test_module "Sequence.merge*" =
         [ Both (2, 2); Left 1; Right 3 ]
     ;;
 
-    let test_merge_semantics ~merge ~normalize_list =
+    let test_merge_semantics ~merge ~(normalize_list : _ -> compare:(_ -> _ -> _) -> _) =
       Base_quickcheck.Test.run_exn
         (module struct
           module Deduped_and_sorted_int_list = struct
@@ -493,7 +493,7 @@ let%expect_test "fold_m" =
     s12345
     ~init:[]
     ~f:(fun acc n ->
-      Simple_monad.bind Simple_monad.step ~f:(fun () -> Simple_monad.return (n :: acc)))
+    Simple_monad.bind Simple_monad.step ~f:(fun () -> Simple_monad.return (n :: acc)))
   |> printf !"%{sexp: int list Simple_monad.t}\n";
   [%expect {| (Step (Step (Step (Step (Step (Return (5 4 3 2 1))))))) |}]
 ;;
@@ -636,12 +636,24 @@ let%test_unit _ =
     ~expect:true
 ;;
 
+let%expect_test "findi" =
+  Base_quickcheck.Test.run_exn
+    (module struct
+      type t = int option list * (int -> int -> bool) [@@deriving quickcheck, sexp_of]
+    end)
+    ~f:(fun (option_list, f) ->
+      let sequence = option_list |> of_list |> filter_opt in
+      [%test_result: (int * int) option]
+        (findi sequence ~f)
+        ~expect:(List.findi (to_list sequence) ~f));
+  [%expect {| |}]
+;;
+
 let%expect_test _ =
   let xs = init 3 ~f:Fn.id |> Generator.of_sequence in
   let ( @ ) xs ys = Generator.bind xs ~f:(fun () -> ys) in
   xs @ xs @ xs @ xs @ xs |> Generator.run |> [%sexp_of: int t] |> print_s;
-  [%expect {|
-    (0 1 2 0 1 2 0 1 2 0 1 2 0 1 2) |}]
+  [%expect {| (0 1 2 0 1 2 0 1 2 0 1 2 0 1 2) |}]
 ;;
 
 let%test_module "group" =
@@ -695,7 +707,8 @@ let%test_module "Caml.Seq" =
         1
         2
         3
-        4 |}]
+        4
+        |}]
     ;;
 
     let%expect_test "to_seq" =
@@ -704,7 +717,8 @@ let%test_module "Caml.Seq" =
         1
         2
         3
-        4 |}]
+        4
+        |}]
     ;;
   end)
 ;;
