@@ -87,44 +87,3 @@ let%test_module "test_compare" =
     let%test _ = Uchar.(compare max_value min_value) = 1
   end)
 ;;
-
-let%expect_test "utf8_byte_length" =
-  let test codepoint =
-    let uchar = Uchar.of_scalar_exn codepoint in
-    let utf8 =
-      let buf = Buffer.create 4 in
-      Uutf.Buffer.add_utf_8 buf uchar;
-      Buffer.contents buf
-    in
-    let computed_byte_length = Uchar.utf8_byte_length uchar in
-    if computed_byte_length <> String.length utf8
-    then
-      print_cr
-        [%here]
-        [%message
-          "utf8_byte_length does not match encoded string"
-            (computed_byte_length : int)
-            ~actual_byte_length:(String.length utf8 : int)
-            (utf8 : string)]
-    else print_s [%sexp (computed_byte_length : int)]
-  in
-  test (Char.to_int 'A');
-  [%expect {| 1 |}];
-  test (* Copyright symbol *) 0x00a9;
-  [%expect {| 2 |}];
-  test (* Check mark *) 0x2713;
-  [%expect {| 3 |}];
-  test (* Cuneiform Sign A *) 0x12000;
-  [%expect {| 4 |}];
-  (* Sanity check: all of ASCII fits in one byte *)
-  for i = 0 to 127 do
-    let char = Char.of_int_exn i in
-    require_equal
-      [%here]
-      (module Int)
-      Uchar.(utf8_byte_length (of_char char))
-      1
-      ~if_false_then_print_s:(lazy [%message "counterexample" (char : char)])
-  done;
-  [%expect {| |}]
-;;
